#!/usr/bin/perl
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#-----------------------------------------------------------------------
#
# @version 1.3 - 12/01/24 01:38:51
#
# @brief
#
# Change activities:
#     - 2004-03-16: Initial version
#
#-----------------------------------------------------------------------

package Ermm::CLI::ChAi;
use constant DIGEST => do{ eval{ require Digest }};
use Getopt::Long;
use Term::ReadLine;
use Ermm::CLI::Conf;
use Ermm::CLI::Main;

$synopsis = "chai <ainame> -app <appname> [-disable | -enable | [ -am <method> -ad <data>] | -admin | -app <name> | -nopc | -pc | -noadmin | -desc <desc> | -prio <num> | -help, -? | -match <c> | -v]";

#
# change application instance
#
sub chai {
    my ($i, $result, $method, @data, $admin, $privChange, $enable, $disable);
    my ($response, $match, $application, $standard, $prio);
    my $help         = 0;
    my $sendString   = "";
    my $matchString  = "";
    my $descString;

    #
    # process command line options
    #
    $result = GetOptions ("v"        => \$Ermm::CLI::Conf::verbose, # flag
                          "disable"  => \$disable,       # flag
                          "enable"   => \$enable,        # flag
                          "am=s"     => \$method,        # string
                          "ad=s@"    => \@data,          # string
                          "admin!"   => \$admin,         # flag
                          "app=s"    => \$application,   # string
                          "pc!"      => \$privChange,    # flag
                          "help|?"   => \$help,          # flag
                          "desc=s"   => \$descString,    # string
                          "match=s"  => \$match,         # string
                          "prio=i"   => \$prio);         # numeric

    #
    # check if we should print usage infos
    #
    if ($result != 1 || $help == 1) {
        Ermm::CLI::Main::printtext('_USAGE_CHAI', $synopsis);
        exit 1;
    }

    #
    # check if there are any arguments
    #
    if (!(($#ARGV && $application) || $match) || !($disable || $enable || $method || @data || defined $admin || defined $privChange || defined $descString || $prio)){
        Ermm::CLI::Main::printtext('EMM7010E');
        Ermm::CLI::Main::printtext('_USAGE_CHAI', $synopsis);
        exit 1;
    }

    #
    # add ai names to match clause
    #
    foreach $i (1 .. $#ARGV) {
        $matchString .= "regex( \"^$ARGV[$i](\:[0-9].*)?\$\" AI.\"AIName\" )";
    }

    if ($#ARGV > 0) {
        $matchString = "or($matchString)";
    }

    #
    # add match clause if specified
    #
    if ($match) {
        $matchString = "and($matchString $match)";
    }

    #
    # add application restriction if specified
    #
    if ($application) {
        $matchString = "and($matchString streq(AI.\"ApplicationName\" ".
            "\"$application\"))";
    }

    if (length($matchString) > 0) {
        $matchString = "match[$matchString]";
    }
    else {
        Ermm::CLI::Main::printtext('EMM7010E');
        Ermm::CLI::Main::printtext('_USAGE_CHAI', $synopsis);
        exit 1;
    }

    #
    # build send string
    #
    $sendString = "attribute task[\"$Ermm::CLI::Conf::taskId\"]";

    if ($enable) {
        $sendString .= " set[AI.\"AIDisabled\" \"false\"]";
    }
    elsif ($disable) {
        $sendString .= " set[AI.\"AIDisabled\" \"true\"]";
    }

    if(defined $descString) {
        $sendString .= " set[AI.\"AIDescription\" \"$descString\"]";
    }

    if (defined $admin && $admin == 1) {
        $sendString .= " set[AI.\"Privilege\" \"ADMINISTRATOR\"]";
    }
    elsif (defined $admin && $admin == 0) {
        $sendString .= " set[AI.\"Privilege\" \"STANDARD\"]";
    }
    if (defined $privChange && $privChange == 1) {
        $sendString .= " set[AI.\"PrivilegeChangeable\" \"true\"]";
    }
    elsif (defined $privChange && $privChange == 0) {
        $sendString .= " set[AI.\"PrivilegeChangeable\" \"false\"]";
    }

    if ($method) {
      if (!(lc($method) =~ m/^(none|password|pwenc)$/)) {
            Ermm::CLI::Main::printtext('EMM7124E', $method);
            Ermm::CLI::Main::printtext('_USAGE_CHAI', $synopsis);
            exit 1;
      }
      if (!(lc($method) =~ m/^(none)$/)) {
        if (@data) {
          if (!DIGEST && (lc($method) =~ m/^(pwenc)$/)) {
            die Ermm::CLI::Main::gettext('EMM7004E');
            exit 1;
          }
        }
        else {
          Ermm::CLI::Main::printtext('_USAGE_CHAI', $synopsis);
          exit 1;
        }
      }
    $sendString .= " set[AI.\"AuthenticationMethod\" \"$method\"]";
    }

    if (@data) {
      my $adata = pop(@data);
      if (length($adata) > 0) {
        if ($method) {
          if ($method =~ m/^pwenc$/i) {
            if (DIGEST) {
              my $sha1 = Digest->new("MD5");
              $sha1->add($adata);
              $adata = $sha1->b64digest;
              }
            else {
              die Ermm::CLI::Main::gettext('EMM7004E');
            }
          }
        }
        else {
          Ermm::CLI::Main::printtext('_USAGE_CHAI', $synopsis);
          exit 1;
        }
      }
      $sendString .= " set[AI.\"AuthenticationData\" \"".$adata."\"]";
    }

    if ($prio) {
        if ($prio > 1000 || $prio < 0) {
            Ermm::CLI::Main::printtext('EMM7122E', $prio);
            Ermm::CLI::Main::printtext('_USAGE_CHAI', $synopsis);
            exit 1;
        }
        $sendString .= " set[AI.\"DefaultPriority\" \"$prio\"]";
    }

    $sendString .= " $matchString;\n";

    #
    # connect to TSLM server
    #
    Ermm::CLI::Main::connect();

    #
    # send command to server
    #
    Ermm::CLI::Main::send($sendString);

    #
    # receive response from server
    #
    $response = Ermm::CLI::Main::getResponse();

}
1;
