#!/usr/bin/perl -w
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#-----------------------------------------------------------------------
#
# @version 1.3 - 12/01/24 01:58:21
#
# @brief
#
# Change activities:
#     - 2005-05-19: Initial version
#
#-----------------------------------------------------------------------

package Ermm::CLI::Email;
use Getopt::Long;
use Term::ReadLine;
use Ermm::CLI::Conf;
use Ermm::CLI::Main;

if ($^O =~ /.*win32.*/i) {
  require Win32::Process;
}

$synopsis = "email -to <toaddress> [ -level <level> | -msgid <id> | -sendmail <cmd> ]";

sub email {

    #
    # process command line options
    #
    my ($to, $help);
    my $level = "";
    my $arglevel = "";
    my $sendmail = "/usr/sbin/sendmail";
    my @sevorder = ( 'emergency', 'alert', 'critical', 'error', 'warning',
                     'notice', 'information', 'debug', 'developer');

    $result = GetOptions ("v"          => \$Ermm::CLI::Conf::verbose,
                          "to=s"       => \$to,
                          "level=s"    => \$arglevel,
                          "msgid=s"    => \@argmsgid,
                          "sendmail=s" => \$sendmail,
                          "help|?"     => \$help);

    #
    # check if we should print usage infos
    #
    if ($result != 1 || $help == 1) {
        Ermm::CLI::Main::printtext('_USAGE_EMAIL', $synopsis);
        exit 1;
    }

    if (!$to || !$sendmail) {
        Ermm::CLI::Main::printtext('EMM7010E');
        Ermm::CLI::Main::printtext('_USAGE_EMAIL', $synopsis);
        exit 1;
    }

    foreach (@sevorder) {
        $level= $_ if $arglevel eq $_;
    }
    if ($arglevel ne "" && $level eq "") {
        #
        # the level argument is nonsens. Probably a syntax mistake.
        #
        Ermm::CLI::Main::printtext('EMM7010E');
        Ermm::CLI::Main::printtext('_USAGE_EMAIL', $synopsis);
        exit 1;
    }
    if ($arglevel eq "") {
        #
        # Don't check based on severity level.
        #
        @sevorder = ();
    }

    my $pid;
    if ($^O =~ /.*win32.*/i) {
      $pid = Win32::Process::GetCurrentProcessID();
    } else {
      $pid = getppid();
    }
    my $sendString = "register task[\"console:$pid\"] ".
        "type[\"event\" \"command:message\" \"always\"];\n";

    Ermm::CLI::Main::connect();
    Ermm::CLI::Main::send($sendString);
    Ermm::CLI::Main::getResponse();

    my $cancelled = 0;
    while(!$cancelled) {
        my $notif = Ermm::CLI::Main::getNotification();
        my ($sec, $min, $hour, $mday, $mon, $year) = gmtime();
        my $t = sprintf("%04d-%02d-%02dT%02d:%02d:%02d",
                        $year+1900, $mon+1, $mday, $hour, $min, $sec);
        my $loctext;
        my $flagsend = 0;
        #
        # match message responses
        #
        if ($notif =~ /.*severity\s*\[\s*\\\"(\w+)\\\".*loctext\s*\[\s*\\\"([^\\]+).*/) {
            $severity = $1;
            $loctext = $2;
            #
            # Check message levels in order of severity.
            # Identify the severity level of the message.
            #
            foreach (@sevorder) {
                if ($severity eq $_) {
                    $flagsend = 1;
                }
                if ($level eq $_) {
                    #
                    # message level threshold reached.
                    # Don't process this message any further.
                    #
                    last;
                }
            }
            #
            # Compare message id with messages in argument argmssgid
            #
            $loctext =~ /^(\S+):/;
            foreach (@argmsgid) {
                $flagsend = 1 if $1 eq $_;
            }

        }
        elsif ($notif =~ /\s*response\s+.*cancelled.*/) {
            Ermm::CLI::Main::printtext('CLICONSMSG1001I', $t);
            $cancelled = 1;
        }
        if ($flagsend) {
            sendmail('sendmail' => $sendmail, 'to' => $to,
                     'subject' => $loctext, 'body' => $loctext);
        }
    }
}

# Send mail using 'sendmail'.
# Arguments: sendmail (required), to (required), replyto (optional),
#            subject (optional), body (optional)
sub sendmail {
    my %args=@_;

    unless ($args{to}) {
        Ermm::CLI::Main::printtext('EMM7140E');
        return;
    }

    my $ok = open(my $SENDMAIL, "|$args{sendmail} $args{to}");

    unless ($ok) {
        Ermm::CLI::Main::printtext('EMM7141E', $args{sendmail});
        return;
    }

    print $SENDMAIL "To: $args{to}\n";
    print $SENDMAIL "Reply-To: $args{replyto}\n" if $args{replyto};
    print $SENDMAIL "Subject: $args{subject}\n" if $args{subject};
    print $SENDMAIL "\n".messageContent()."\n";
    print $SENDMAIL $args{body} if $args{body};

    close($SENDMAIL);
}

# Generic message content of the Email notification
sub messageContent {

    my ($sec, $min, $hour, $mday, $mon, $year) = gmtime();
    my $timestamp = sprintf("%04d-%02d-%02d T%02d:%02d:%02d",
                            $year+1900, $mon+1, $mday, $hour, $min, $sec);

    my $message = "IBM Tape System Library Manager\n";
    $message .= "Automated Email Notification from $timestamp\n\n";

    return $message;
}

1;
