#!/usr/bin/perl -w
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#-----------------------------------------------------------------------
#
# @version 1.3 - 12/01/24 02:00:02
#
# @brief Health check module for ermmtool
#
# Change activities:
#     - 2005-10-06: Initial version
#
#-----------------------------------------------------------------------

package Ermm::CLI::Health;
use Getopt::Long;
use Term::ReadLine;
use Ermm::CLI::Conf;
use Ermm::CLI::Main;

$synopsis = "health [ -v | -all | -mr | -mm | -lr | -lo | -da | -do | -dm | -co | -sa | -dga | -cga |-hdr [on|off] | -help, -? | -fmt [xml|stanza|delim=c]]";

#
# Setup configuration variables.
#
my ($origClArgs) = "@ARGV" =~ /(.*?) (?:-m 1 )?health.*/;

my $prefixdir = "/opt/IBM/ermm/";
my $tool = "$0 ";

my $ok = "ok";
my $failed = "failed";

#
# list volumes
#
sub health {
    my ($i, $longOutput, $shortOutput, $ermmOutput, $result);
    my ($response, $match, $order, $number, $application, $regex, $pcl);
    my $help = 0;
    my $reportmode = "";
    my $reportString = "";
    my $flag = "";
    my %health;
    my $alltests;

    #
    # get defaults from configuration
    #
    my $format = $Ermm::CLI::Conf::format;
    my $delimiter = $Ermm::CLI::Conf::delimiter;
    my $header = $Ermm::CLI::Conf::header;

    #
    # the classname for xml output
    #
    my $className = "Health";

    #
    # attribute names on headline
    #
    my @propNames = ("Test name", "Result");

    #
    # attribute types
    #
    my @typeNames = ("string", "string");

    #
    # process command line options
    #
    $result = GetOptions ("l"        => \$longOutput,    # flag
                          "s"        => \$shortOutput,   # flag
                          "x"        => \$ermmOutput,    # flag
                          "v"        => \$Ermm::CLI::Conf::verbose, # flag
                          "fmt=s"    => \$format,        # string
                          "help|?"   => \$help,          # flag
                          "hdr=s"    => \$header,        # string
                          "mr!"      => \$health{'mr'},  # flag
                          "mm!"      => \$health{'mm'},  # flag
                          "lr!"      => \$health{'lr'} , # flag
                          "lo!"      => \$health{'lo'} , # flag
                          "da!"      => \$health{'da'},  # flag
                          "do!"      => \$health{'do'},  # flag
                          "dm!"      => \$health{'dm'} , # flag
                          "co!"      => \$health{'co'},  # flag
                          "sa!"      => \$health{'sa'},  # flag
                          "dga!"     => \$health{'dga'}, # flag
                          "cga!"     => \$health{'cga'}, # flag
                          "all!"     => \$alltests,      # flag
                          "match=s"  => \$match,         # string
                          "order=s"  => \$order,         # string
                          "number=s" => \$number);       # string

    # Command behavior:
    #   when no parameters are given run all tests.
    #   the all parameter is the default for unset tests
    #   when no tests are set to run then set all parameter automatically

    # with no test parameter set to run set default for undefined tests to run
    if ($alltests eq "") {
        $alltests = (!grep $_ != 0 ,%health) ? 1 : 0;
    }

    #
    # test if run localy and cancel tests if not
    #
    if (!($Ermm::CLI::Conf::host eq 'localhost')) {
        $health{'mr'} = 0 if $health{'mr'} eq "";
        $health{'lr'} = 0 if $health{'lr'} eq "";
    }

    #
    # with no test parameter set to run set default for undefined tests to run
    #
    foreach (keys %health) {
        if ($health{$_} eq "") {
            $health{$_} = $alltests;
        };
    };

    #
    # check if we should print usage infos
    #
    if ($result != 1 || $help == 1) {
        Ermm::CLI::Main::printtext('_USAGE_HEALTH', $synopsis);
        exit 1;
    }

    #
    # check header option
    #
    if ($header eq "on" || $header eq "yes") {
        $header = 1;
    }

    #
    # get delimiter from -fmt option
    #
    if ($format =~ /^delim=(.*)/) {
        $format = "delim";
        $delimiter = $1;
    }

    #
    # center ok for format delim with header
    #
    if (($format =~ /^delim/) and ($header ne "off")) {
        $ok= "  ok";
    }

    #
    # call tests
    #
    my @table;
    testmr_mmrunning(\@table) if $health{'mr'};
    testmm_mmconnect(\@table) if $health{'mm'};
    testlr_lmrunning(\@table) if $health{'lr'};
    testlo_libraryon(\@table) if $health{'lo'};
    testda_drives(\@table) if $health{'da'};
    testdo_drives(\@table) if $health{'do'};
    testdm_drives(\@table) if $health{'dm'};
    testco_cart(\@table) if $health{'co'};
    testsa_cart(\@table) if $health{'sa'};
    testdga_dga(\@table) if $health{'dga'};
    testcga_cga(\@table) if $health{'cga'};

    # print result
    Ermm::CLI::Main::printResults($format, $delimiter, $header, $className,
                                  \@propNames, \@typeNames, \@table);
}

#
# check pid
#
sub testmr_mmrunning {
    my ($table) = @_;

    # read supposed MediaManager pid
    open PIDH, "<${prefixdir}run/mm.pid";
    my ($pid)= <PIDH> =~ /(\d*)/;
    close PIDH;

    # go through pid list and search for our MediaManager processes
    open PIDSH, "ps -e|";
    my @found = grep /^ *$pid .*(MediaManager|ermmStart)$/, <PIDSH>;
    close PIDSH;

    ${$table}[++$#$table][0] = "Find process of MediaManager";
    ${$table}[$#$table][1] = (@found == 1) ? $ok : $failed;
}

#
# connect to TSLM server
#
sub testmm_mmconnect {
    my ($table) = @_;

    #
    # surpress output
    #
    local $^W = 0;
    my $stdout = select CLOSEDHANDLE;

    #
    # connect to mm
    #
    my $ret = eval{ Ermm::CLI::Main::connect(); 0;};

    #
    # restore output
    #
    $^W = 1;
    select $stdout;

    ${$table}[++$#$table][0] = "Connection to MediaManager";
    ${$table}[$#$table][1] = ($ret eq "0") ? $ok : $failed;
}

#
# check lm pid
#
sub testlr_lmrunning {
    my ($table) = @_;
    my $lmconfdir = $prefixdir . "conf/";

    # read supposed Library Manager
    opendir(CONFH, $lmconfdir) || die Ermm::CLI::Main::gettext('EMM7022E', $lmconfdir);
#	"Cannot open directory $lmconfdir\n";
    my @lmfiles = grep /^lm-.*\.xml$/, readdir(CONFH);
    closedir CONFH;

    # isolate lm name of filename
    map s/^lm-(.*)\.xml$/\1/, @lmfiles;

    # for each Library Manager
    foreach my $lm (@lmfiles) {

        # read supposed Library Manager pids
        open PIDH, "</opt/IBM/ermm/run/lm-$lm.pid";
        my ($pid)= <PIDH> =~ /(\d*)/;
        close PIDH;

        # go through pid list and search for our MediaManager processes
        open PIDSH, "ps -e|";
        my @found = grep /^ *$pid .*LibraryManager$/, <PIDSH>;
        close PIDSH;

        ${$table}[++$#$table][0] = "Find process of LibraryManager $lm";
        ${$table}[$#$table][1] = (@found == 1) ? $ok : $failed;
    }
}

#
# Librarys available in Media Manager
#
sub testlm_library {
    my ($table) = @_;
    my $ret = 0;

    #
    # call lscg with open | to search for empty scratch pools
    #
    open( my $FH, "$tool $origClArgs -p \"$Ermm::CLI::Conf::password\" -m 0 lslm -fmt delim=\" \" 2>&1 |");

    #
    # search for line ========= followed by data
    #
    foreach (<$FH>) {
        $ret = 1 if /^(=+)$/ && /^.+$/;
    }

    close $FH;

    ${$table}[++$#$table][0] = "Librarys available";
    ${$table}[$#$table][1] = $ret ? $ok : $failed;
}

#
# Librarys online in Media Manager
#
sub testlo_libraryon {
    my ($table) = @_;
    my $ret = 0;

    #
    # call lscg with open | to search for empty scratch pools
    #
    open( FH, "$tool $origClArgs -p \"$Ermm::CLI::Conf::password\" -m 0 lslib -fmt delim=\" \" ".
          "-match 'streq(Library.\"LibraryOnline\" \"true\")' 2>&1 |");

    #
    # search for line ========= followed by data
    #
    foreach (<FH>) {
        $ret = 1 if /^(=+)$/ && /^.+$/;
    }

    close FH;

    ${$table}[++$#$table][0] = "Librarys online";
    ${$table}[$#$table][1] = $ret ? $ok : $failed;
}

#
# Drives available
#
sub testda_drives {
    my ($table) = @_;
    my $ret = 0;

    #
    # call lsdrive with open | to search for drives reported by library
    #
    open( FH, "$tool $origClArgs -p \"$Ermm::CLI::Conf::password\" -m 0 lsdrive -fmt delim=\" \" 2>&1 |");

    #
    # search for line ========= followed by data
    #
    foreach (<FH>) {
        $ret = 1 if /^(=+)$/ && /^.+$/;
    }

    close FH;

    ${$table}[++$#$table][0] = "Drives available";
    ${$table}[$#$table][1] = $ret ? $ok : $failed;
}

#
# Drives online
#
sub testdo_drives {
    my ($table) = @_;
    my $ret = 0;

    #
    # call lscg with open | to search for empty scratch pools
    #
    open( FH, "$tool $origClArgs -p \"$Ermm::CLI::Conf::password\" -m 0 lsdrive -fmt delim=\" \" ".
          "-match 'streq(Drive.\"DriveOnline\" \"true\")' 2>&1 |");

    #
    # search for line ========= followed by data
    #
    foreach (<FH>) {
        $ret = 1 if /^(=+)$/ && /^.+$/;
    }

    close FH;

    ${$table}[++$#$table][0] = "Drives online";
    ${$table}[$#$table][1] = $ret ? $ok : $failed;
}

#
# Drives reported by hdm
#
sub testdm_drives {
    my ($table) = @_;
    my $ret = 0;

    #
    # call ls with open | to search for drives reported by hdm
    #
    open( FH, "$tool $origClArgs -p \"$Ermm::CLI::Conf::password\" -m 0 lsdm -fmt delim=\" \" ".
          "-match 'or \(streq\(DM.\"DMStateSoft\" \"ready\"\) streq\(DM.\"DMStateSoft\" \"present\"\)\)' 2>&1 |");

    #
    # search for line ========= followed by data
    #
    foreach (<FH>) {
        $ret = 1 if /^(=+)$/ && /^.+$/;
    }

    close FH;

    ${$table}[++$#$table][0] = "Drives found on clients";
    ${$table}[$#$table][1] = $ret ? $ok : $failed;
}

#
# Cartridges online
#
sub testco_cart {
    my ($table) = @_;
    my $ret = 0;

    #
    # call lscg with open | to search for empty scratch pools
    #
    open( FH, "$tool $origClArgs -p \"$Ermm::CLI::Conf::password\" -m 0 lscart -fmt delim=\" \" ".
          "-number 1 2>&1 |");

    #
    # search for line ========= followed by data
    #
    foreach (<FH>) {
        $ret = 1 if /^(=+)$/ && /^.+$/;
    }

    close FH;

    ${$table}[++$#$table][0] = "Cartridges online";
    ${$table}[$#$table][1] = $ret ? $ok : $failed;
}

#
# Scratch Cartridges available in each scratch pool
#
sub testsa_cart {
    my ($table) = @_;
    my $ret = 0;

    #
    # call lscg with open | to search for empty scratch pools
    #
    open( FH, "$tool $origClArgs -p \"$Ermm::CLI::Conf::password\" -m 0 lscg -fmt delim=\" \" ".
          "-match 'streq(CartridgeGroup.\"IsScratchPool\" \"true\")' 2>&1 |");
    my @lines = <FH>;
    close FH;

    #
    # search for Scratchpool = true and Cartridges = 0
    #
    foreach (@lines) {
        /^\S+\s+\d+\s+(\S+)\s+(\d+)\s*/;
        $ret = 1 if $1 eq "true"; # only ok when we have a scratch pool
        if (($1 eq "true") && ($2 eq "0")) {
            $ret = 0;
            last;
        }
    }

    ${$table}[++$#$table][0] = "Scratch Cartridges available";
    ${$table}[$#$table][1] = $ret ? $ok : $failed;
}

#
# Drive Group Application entered
#
sub testdga_dga {
    my ($table) = @_;
    my $ret = 0;

    #
    # call lscg with open | to search for empty scratch pools
    #
    open( FH, "$tool $origClArgs -p \"$Ermm::CLI::Conf::password\" -m 0 lsdga -fmt delim=\" \" 2>&1 |");

    #
    # search for line ========= followed by data
    #
    foreach (<FH>) {
        $ret = 1 if /^(=+)$/ && /^.+$/;
    }

    close FH;

    ${$table}[++$#$table][0] = "Drive Group Application entered";
    ${$table}[$#$table][1] = $ret ? $ok : $failed;
}

#
# Cartridge Group Application entered
#
sub testcga_cga {
    my ($table) = @_;
    my $ret = 0;

    #
    # call lscg with open | to search for empty scratch pools
    #
    open( FH, "$tool $origClArgs -p \"$Ermm::CLI::Conf::password\" -m 0 lscga -fmt delim=\" \" 2>&1 |");

    #
    # search for line ========= followed by data
    #
    foreach (<FH>) {
        $ret = 1 if /^(=+)$/ && /^.+$/;
    }

    close FH;

    ${$table}[++$#$table][0] = "Cartridge Group Application entered";
    ${$table}[$#$table][1] = $ret ? $ok : $failed;
}

1;
