#!/usr/bin/perl -w
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#-----------------------------------------------------------------------
#
# @version 1.3 - 12/01/24 01:48:16
#
# @brief
#
# Change activities:
#     - 2004-03-16: Initial version
#
#-----------------------------------------------------------------------

package Ermm::CLI::LsApp;
use Getopt::Long;
use Term::ReadLine;
use Ermm::CLI::Conf;
use Ermm::CLI::Main;

$synopsis = "lsapp [-l | -s | -x | -v | -hdr [on|off] | -help, -? | -match <c> | -order <c> | -number <c> | -fmt [xml|stanza|delim=c]] [<appname>]";

#
# list application
#
sub lsapp {
    my ($i, $longOutput, $shortOutput, $ermmOutput, $result);
    my ($response, $match, $order, $number);
    my $help = 0;
    my $matchString = "";
    my $orderString = "";
    my $numberString = "";
    my $reportmode = "";
    my $reportString = "";
    my $flag = "";

    #
    # get defaults from configuration
    #
    my $format = $Ermm::CLI::Conf::format;
    my $delimiter = $Ermm::CLI::Conf::delimiter;
    my $header = $Ermm::CLI::Conf::header;

    #
    # the classname for xml output
    #
    my $className = "Application";

    #
    # attributes listed by default
    #
    my $defaultReport =
        " Application.\"ApplicationName\" ".
        " Application.\"ApplicationDisabled\" ".
        " Application.\"AllowRemoteMount\" ".
        " Application.\"ApplicationMaxDisconnectTimeout\" ";

    #
    # attributes listed if -l option given
    #
    my $longReport = $defaultReport .
      " Application.\"ApplicationDescription\"";

    #
    # attributes listed if -s option given
    #
    my $shortReport = "Application.\"ApplicationName\"";

    #
    # attribute names on headline
    #
    my @propNames = ("Name", "Disabled", "AllowRemoteMount", "MaxDisconnectTimeout", "Description");

    #
    # attribute types
    #
    my @typeNames = ("string", "string", "string", "int", "string");

    #
    # process command line options
    #
    $result = GetOptions ("l"        => \$longOutput, # flag
                          "s"        => \$shortOutput,# flag
                          "x"        => \$ermmOutput, # flag
                          "v"        => \$Ermm::CLI::Conf::verbose, # flag
                          "fmt=s"    => \$format, # string
                          "help|?"   => \$help,   # flag
                          "hdr=s"    => \$header, # string
                          "match=s"  => \$match, # string
                          "order=s"  => \$order, # string
                          "number=s" => \$number); # string

    #
    # check if we should print usage infos
    #
    if ($result != 1 || $help == 1) {
        Ermm::CLI::Main::printtext('_USAGE_LSAPP', $synopsis);
        exit 1;
    }

    #
    # check header option
    #
    if ($header eq "on" || $header eq "yes") {
        $header = 1;
    }

    #
    # get delimiter from -fmt option
    #
    if ($format =~ /^delim=(.*)/) {
        $format = "delim";
        $delimiter = $1;
    }

    #
    # add application names to match clause
    #
    foreach $i (1 .. $#ARGV) {
        $matchString .= "streq(Application.\"ApplicationName\" \"$ARGV[$i]\")";
    }

    if ($#ARGV > 0) {
        $matchString = "or($matchString)";
    }

    #
    # add match clause if specified
    #
    if ($match) {
        $matchString = "and($matchString $match)";
    }

    if ($order) {
        $orderString = "$order";
    }
    else {
        $orderString = "strlohi(Application.\"ApplicationName\")";
    }

    if ($number) {
        $numberString = "$number";
        $flag = "q";
    }

    #
    # build report string
    #
    if ($longOutput) {
        $reportString .= $longReport;
    }
    elsif ($shortOutput) {
        $reportString .= $shortReport;
    }
    elsif ($ermmOutput) {
        $reportString .= "$className";
        $reportmode = "namevalue";
    }
    else {
        $reportString .= $defaultReport;
    }

    #
    # connect to TSLM server
    #
    Ermm::CLI::Main::connect();

    #
    # if number is not specified, print total number of objects.
    #
    if (!$flag) {
      my $sendString = Ermm::CLI::Main::composeMMPShow($Ermm::CLI::Conf::taskId,
                                                       $matchString,
                                                       $orderString,
                                                       $numberString,
                                                       $reportString,
                                                       "number");

      Ermm::CLI::Main::send($sendString);
      $response = Ermm::CLI::Main::getResponse();

      #
      # get total number of objects.
      #
      @values = Ermm::CLI::Main::getKeyValueResults($response);

      if ( ($#values < 0) || ($values[0][0] < 1) ) {
          Ermm::CLI::Main::printtext('EMM7009I');
          exit;
      }
    }
    my $records = $values[0][0];
    my $recread = 0;

    #
    # send command to server
    #
    Ermm::CLI::Main::sendMMPShow($Ermm::CLI::Conf::taskId,
                                 $matchString,
                                 $orderString,
                                 $numberString,
                                 $reportString,
                                 $reportmode);

    #
    # receive response from server
    #
    $response = Ermm::CLI::Main::getResponse();

    #
    # Begin loop
    #
    do {
        if ($ermmOutput) {
          #
          # -x option specified; print raw TSLM output
          #
          print $response,"\n";
        }
        else {
          #
          # start normal output processing
          #
          @values = Ermm::CLI::Main::getKeyValueResults($response);

          if ($#values < 0) {
              exit;
          }

          # in case the client attribute ApplicationDisabled is not set, display "false"
          for ($i = 0; $i <= $#values; $i++) {
              if ($values[$i][1] eq "") { $values[$i][1] = "false"; }
          }

          Ermm::CLI::Main::printResults($format, $delimiter, $header, $className,
                                      \@propNames, \@typeNames, \@values);
        }
        $recread += $Ermm::CLI::Conf::objlimit;
        if ($recread >= $records) {
            $flag = "q";
        }
        #
        # Get next part of response
        #
        if (!$flag) {
          Ermm::CLI::Main::nextMMPShowPart();
          #
          # receive response from server
          #
          $response = Ermm::CLI::Main::getResponse();

          #
          # Check if response has values
          #
            if ($response =~ m/text\s*\[.+\]/) {
              if ($Ermm::CLI::Conf::mode) {
                $flag = " ";
              }
              else {
                $flag = "";
              }
            }
            else {
              $flag = "q";
            }

          #
          # Prompt continue or quit
          #
            while ($flag ne "" && $flag ne "q") {
              Ermm::CLI::Main::printtext('CLIMSG_PROMT');
              $flag = <STDIN>;
              chomp $flag;
            }
        }

    }  while (!$flag)
}

1;
