#!/usr/bin/perl -w
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#-----------------------------------------------------------------------
#
# @version 1.3 - 12/01/24 01:48:19
#
# @brief
#
# Change activities:
#     - 2004-03-16: Initial version
#
#-----------------------------------------------------------------------

package Ermm::CLI::LsCart;
use Getopt::Long;
use Term::ReadLine;
use Ermm::CLI::Conf;
use Ermm::CLI::Main;

$synopsis = "lscart [-l | -s | -x | -v | -access |-lib <name> | -group <id> | -regex <match> | -type <id> | -state <state_name> | -hdr [on|off] | -help, -? | -match <c> | -order <c> | -number <c> | -fmt [xml|stanza|delim=c]] [<pcl>]";

#
# list cartridges
#
sub lscart {
    my ($i, $longOutput, $shortOutput, $ermmOutput, $result, $type);
    my ($response, $access, $match, $order, $number, $library, $group, $state, $regex);
    my $help = 0;
    my $matchString = "";
    my $orderString = "";
    my $numberString = "";
    my $reportmode = "";
    my $reportString = "";
    my $flag = "";

    #
    # get defaults from configuration
    #
    my $format = $Ermm::CLI::Conf::format;
    my $delimiter = $Ermm::CLI::Conf::delimiter;
    my $header = $Ermm::CLI::Conf::header;

    #
    # the classname for xml output
    #
    my $className = "Cartridge";

    #
    # attributes listed by default
    #
    my $defaultReport =
        " Cartridge.\"CartridgePCL\" Cartridge.\"CartridgeTypeName\"".
        " Cartridge.\"LibraryName\" Cartridge.\"CartridgeState\"".
        " Cartridge.\"CartridgeGroupName\" Cartridge.\"ApplicationName\"";

    #
    # attributes listed if -l option given
    #
    my $longReport = $defaultReport .
        " Cartridge.\"DeallocGroup\"".
        " Cartridge.\"CartridgeNumberMounts\"".
        " Cartridge.\"CartridgeTimeMountedLast\"".
        " Cartridge.\"CartridgeTimeMountedTotal\"";

    #
    # attributes listed if -s option given
    #
    my $shortReport = "Cartridge.\"CartridgePCL\"";

    #
    # attribute names on headline
    #
    my @propNames = ("PCL", "Type", "Library", "State",
                     "Group", "Application", "DeallocGroup",
                     "Mounts", "MountedLast", "TimeMountedTotal");
    #
    # attribute types
    #
    my @typeNames = ("string", "string", "string", "string",
                     "string", "string", "string", "int",
                     "dateTime", "int");

    #
    # process command line options
    #
    $result = GetOptions ("l"        => \$longOutput, # flag
                          "s"        => \$shortOutput,# flag
                          "x"        => \$ermmOutput, # flag
                          "v"        => \$Ermm::CLI::Conf::verbose, # flag
                          "access"   => \$access, # flag
                          "fmt=s"    => \$format, # string
                          "help|?"   => \$help,   # flag
                          "hdr=s"    => \$header, # string
                          "lib=s"    => \$library, # string
                          "group=s"  => \$group, # string
                          "state=s"  => \$state, # string
                          "regex=s"  => \$regex, # string
                          "type=s"   => \$type, # string
                          "match=s"  => \$match, # string
                          "order=s"  => \$order, # string
                          "number=s" => \$number); # string

    #
    # check if we should print usage infos
    #
    if ($result != 1 || $help == 1) {
        Ermm::CLI::Main::printtext('_USAGE_LSCART', $synopsis);
        exit 1;
    }

    #
    # check header option
    #
    if ($header eq "on" || $header eq "yes") {
        $header = 1;
    }

    #
    # get delimiter from -fmt option
    #
    if ($format =~ /^delim=(.*)/) {
        $format = "delim";
        $delimiter = $1;
    }

    #
    # add drive names to match clause
    #
    foreach $i (1 .. $#ARGV) {
        $matchString .= "streq(Cartridge.\"CartridgePCL\" \"$ARGV[$i]\")";
    }

    if ($#ARGV > 0) {
        $matchString = "or($matchString)";
    }

    #
    # add match clause if specified
    #
    if ($match) {
        $matchString = "and($matchString $match)";
    }

    #
    # add CartridgeGroupName restriction if specified
    #
    if ($access) {
        $matchString = "and($matchString strne(Cartridge.\"CartridgeGroupName\" \"NoAccess\"))";
    }

    #
    # add library restriction if specified
    #
    if ($library) {
        $matchString = "and($matchString streq(Cartridge.\"LibraryName\" ".
            "\"$library\"))";
    }

    #
    # add group restriction if specified
    #
    if ($group) {
        $matchString = "and($matchString streq(Cartridge.\"CartridgeGroupName\"".
            " \"$group\"))";
    }

    #
    # add state restriction if specified
    #
    if ($state) {
        $matchString = "and($matchString streq(Cartridge.\"CartridgeState\"".
            " \"$state\"))";
    }

    #
    # add type restriction if specified
    #
    if ($type) {
        $matchString = "and($matchString streq(Cartridge.\"CartridgeTypeName\"".
            " \"$type\"))";
    }

    #
    # add PCL restriction if specified
    #
    if (defined $regex) {
        $matchString = "and($matchString regex(\"$regex\"".
            " Cartridge.\"CartridgePCL\"))";
    }

    if ($order) {
        $orderString = "$order";
    }
    else {
        $orderString = "strlohi(Cartridge.\"CartridgePCL\")";
    }

    if ($number) {
        $numberString = "$number";
        $flag = "q";
    }

    #
    # build report string
    #
    if ($longOutput) {
        $reportString .= $longReport;
    }
    elsif ($shortOutput) {
        $reportString .= $shortReport;
    }
    elsif ($ermmOutput) {
        $reportString .= "$className";
        $reportmode = "namevalue";
    }
    else {
        $reportString .= $defaultReport;
    }

    #
    # connect to TSLM server
    #
    Ermm::CLI::Main::connect();

    #
    # if number is not specified, print total number of objects.
    #
    if (!$flag) {
      my $sendString = Ermm::CLI::Main::composeMMPShow($Ermm::CLI::Conf::taskId,
                                                       $matchString,
                                                       $orderString,
                                                       $numberString,
                                                       $reportString,
                                                       "number");

      Ermm::CLI::Main::send($sendString);
      $response = Ermm::CLI::Main::getResponse();

      #
      # get total number of objects.
      #
      @values = Ermm::CLI::Main::getKeyValueResults($response);

      if ( ($#values < 0) || ($values[0][0] < 1) ) {
          Ermm::CLI::Main::printtext('EMM7009I');
          exit;
      }
    }
    my $records = $values[0][0];
    my $recread = 0;

    #
    # send command to server
    #
    Ermm::CLI::Main::sendMMPShow($Ermm::CLI::Conf::taskId,
                                 $matchString,
                                 $orderString,
                                 $numberString,
                                 $reportString,
                                 $reportmode);

    #
    # receive response from server
    #
    $response = Ermm::CLI::Main::getResponse();

    #
    # Begin loop
    #
    do {
        if ($ermmOutput) {
          #
          # -x option specified; print raw TSLM output
          #
          print $response,"\n";
        }
        else {
          #
          # start normal output processing
          #
          @values = Ermm::CLI::Main::getKeyValueResults($response);

          if ($#values < 0) {
              exit;
          }

          Ermm::CLI::Main::printResults($format, $delimiter, $header, $className,
                                      \@propNames, \@typeNames, \@values);
        }
        $recread += $Ermm::CLI::Conf::objlimit;
        if ($recread >= $records) {
            $flag = "q";
        }

        #
        # Get next part of response
        #
        if (!$flag) {
          Ermm::CLI::Main::nextMMPShowPart();
          #
          # receive response from server
          #
          $response = Ermm::CLI::Main::getResponse();

          #
          # Check if response has values
          #
            if ($response =~ m/text\s*\[.+\]/) {
              if ($Ermm::CLI::Conf::mode) {
                $flag = " ";
              }
              else {
                $flag = "";
              }
            }
            else {
              $flag = "q";
            }

          #
          # Prompt continue or quit
          #
            while ($flag ne "" && $flag ne "q") {
              Ermm::CLI::Main::printtext('CLIMSG_PROMT');
              $flag = <STDIN>;
              chomp $flag;
            }
        }

    }  while (!$flag)
}

1;
