#!/usr/bin/perl -w
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#-----------------------------------------------------------------------
#
# @version 1.3 - 12/01/24 01:48:28
#
# @brief
#
# Change activities:
#     - 2004-03-16: Initial version
#
#-----------------------------------------------------------------------

package Ermm::CLI::LsDca;
use Getopt::Long;
use Term::ReadLine;
use Ermm::CLI::Conf;
use Ermm::CLI::Main;
use Time::Local;

$synopsis = "lsdca [-l | -s | -x | -v | -app <name> | -drive <name> | -cart <pcl> | -hdr [on|off] | -help, -? | -match <c> | [-sodd|-soda|-socd|-soca|-soad|-soaa|-sotd|-sota|-order <c>] | -begin <time> | -end <time> | -utc | -incomplete | -number <c> | -fmt [xml|stanza|delim=c]] [<id>]";

#
# list drive cartridge objects
#
sub lsdca {
    my ($i, $longOutput, $shortOutput, $ermmOutput, $result);
    my ($response, $match, $order, $number, $level, $application, $drive, $cart);
    my ($sodd, $soda, $socd, $soca, $soad, $soaa, $sotd, $sota);
    my $help = 0;
    my $matchString = "";
    my $orderString = "";
    my $numberString = "";
    my $reportmode = "";
    my $reportString = "";
    my $flag = "";
    my $utc = 0;
    my $incomplete = 0;
    my $begin;
    my $end;

    #
    # get defaults from configuration
    #
    my $format = $Ermm::CLI::Conf::format;
    my $delimiter = $Ermm::CLI::Conf::delimiter;
    my $header = $Ermm::CLI::Conf::header;

    #
    # the classname for xml output
    #
    my $className = "DriveCartridgeAccess";

    #
    # attributes listed by default
    #
    my $defaultReport =
        " DriveCartridgeAccess.\"ApplicationName\"".
        " DriveCartridgeAccess.\"AIName\"".
        " DriveCartridgeAccess.\"CartridgePCL\"".
        " DriveCartridgeAccess.\"DriveName\"".
        " DriveCartridgeAccess.\"DriveCartridgeAccessTimeMount\"".
        " DriveCartridgeAccess.\"DriveCartridgeAccessTimeUnmount\"";

    #
    # attributes listed if -l option given
    #
    my $longReport = $defaultReport.
        " DriveCartridgeAccess.\"DriveCartridgeAccessByteReadCount\"".
        " DriveCartridgeAccess.\"DriveCartridgeAccessByteWriteCount\"".
        " DriveCartridgeAccess.\"DriveCartridgeAccessHardReadErrorCount\"".
        " DriveCartridgeAccess.\"DriveCartridgeAccessHardWriteErrorCount\"".
        " DriveCartridgeAccess.\"DriveCartridgeAccessPhysicalDrive\"";

    #
    # attributes listed if -s option given
    #
    my $shortReport = "DriveCartridgeAccess.\"DriveCartridgeAccessID\"";

    #
    # attribute names on headline
    #
    my @propNames = ("Application", "AI", "CartridgePCL", "Drive",
                     "TimeMounted", "TimeUnmounted",
                     "BytesRead", "BytesWritten",
                     "ReadErrors", "WriteErrors", "PhysicalDrive");
    #
    # attribute types
    #
    my @typeNames = ("string", "string", "string", "string",
                     "dateTime", "dateTime",
                     "int", "int", "int", "int");

    #
    # process command line options
    #
    $result = GetOptions ("l"         => \$longOutput, # flag
                          "s"         => \$shortOutput,# flag
                          "x"         => \$ermmOutput, # flag
                          "v"         => \$Ermm::CLI::Conf::verbose, # flag
                          "sodd"      => \$sodd, # flag
                          "soda"      => \$soda, # flag
                          "socd"      => \$socd, # flag
                          "soca"      => \$soca, # flag
                          "soad"      => \$soad, # flag
                          "soaa"      => \$soaa, # flag
                          "sotd"      => \$sotd, # flag
                          "sota"      => \$sota, # flag
                          "begin=s"   => \$begin,   # date string
                          "end=s"     => \$end, # date string
                          "incomplete"=> \$incomplete, # flag
                          "utc"       => \$utc,  # flag
                          "fmt=s"     => \$format, # string
                          "help|?"    => \$help,   # flag
                          "hdr=s"     => \$header, # string
                          "app=s"     => \$application, # string
                          "drive=s"   => \$drive, # string
                          "cart=s"    => \$cart, # string
                          "match=s"   => \$match, # string
                          "order=s"   => \$order, # string
                          "number=s"  => \$number); # string

    #
    # check if we should print usage infos
    #
    if ($result != 1 || $help == 1) {
        Ermm::CLI::Main::printtext('_USAGE_LSDCA', $synopsis);
        exit 1;
    }

    #
    # check header option
    #
    if ($header eq "on" || $header eq "yes") {
        $header = 1;
    }

    #
    # set utc format if necessary
    #
    if ($utc) {
        @typeNames = map { s/dateTime/dateTimeUTC/; $_ } @typeNames;
    }

    #
    # get delimiter from -fmt option
    #
    if ($format =~ /^delim=(.*)/) {
        $format = "delim";
        $delimiter = $1;
    }

    if ($shortOutput) {
        #
        # attribute names on headline
        #
        @propNames = ("ID");

        #
        # attribute types
        #
        @typeNames = ("string");

    }

    #
    # add task id to match clause
    #
    foreach $i (1 .. $#ARGV) {
        $matchString .= "streq(DriveCartridgeAccess.\"DriveCartridgeAccessID\" \"$ARGV[$i]\")";
    }

    if ($#ARGV > 0) {
        $matchString = "or($matchString)";
    }

    #
    # add match clause if specified
    #
    if ($match) {
        $matchString = "and($matchString $match)";
    }

    #
    # add application restriction if specified
    #
    if ($application) {
        $matchString = "and($matchString streq(DriveCartridgeAccess.".
            "\"ApplicationName\" \"$application\"))";
    }

    #
    # add drive restriction if specified
    #
    if ($drive) {
        $matchString = "and($matchString streq(DriveCartridgeAccess.".
            "\"DriveName\" \"$drive\"))";
    }

    #
    # add cartridge restriction if specified
    #
    if ($cart) {
        $matchString = "and($matchString streq(DriveCartridgeAccess.".
            "\"CartridgePCL\" \"$cart\"))";
    }

    #
    # Don't display incomplete records unless specified
    #
    if ($end or !($matchString or $begin or $incomplete)) {
        my $begindate = "1900-01-01 00:00:01";
        $matchString = "and($matchString strge(".
          "DriveCartridgeAccess.\"DriveCartridgeAccessTimeUnmount\" \"$begindate\"))";
    }

    #
    # Display incomplete records
    #
    if ($incomplete) {
        my $incompletedate = "1900-01-01 00:00:00";
        $matchString = "and($matchString streq(".
          "DriveCartridgeAccess.\"DriveCartridgeAccessTimeUnmount\" \"$incompletedate\"))";
    }

    $begin = Date2DB2($begin, $utc) if $begin;
    if($begin) {
        $matchString = "and($matchString strge(".
            "DriveCartridgeAccess.\"DriveCartridgeAccessTimeUnmount\" \"$begin\"))";
    }
    $end = Date2DB2($end, $utc) if $end;
    if($end) {
        $matchString = "and($matchString strlt(".
            "DriveCartridgeAccess.\"DriveCartridgeAccessTimeUnmount\" \"$end\"))";
    }

    if ($order) {
        $orderString = "$order";
    }
    else {
        if ($sodd) {
            $orderString = "strhilo(DriveCartridgeAccess.\"DriveName\")";
        }
        elsif($soda) {
            $orderString = "strlohi(DriveCartridgeAccess.\"DriveName\")";
        }
        elsif ($socd) {
            $orderString = "strhilo(DriveCartridgeAccess.\"CartridgePCL\")";
        }
        elsif($soca) {
            $orderString = "strlohi(DriveCartridgeAccess.\"CartridgePCL\")";
        }
        elsif ($soad) {
            $orderString = "strhilo(DriveCartridgeAccess.\"ApplicationName\")";
        }
        elsif($soaa) {
            $orderString = "strlohi(DriveCartridgeAccess.\"ApplicationName\")";
        }
        elsif($sotd) {
            $orderString = "numhilo(DriveCartridgeAccess.\"DriveCartridgeAccessTimeUnmount\")";
        }
        elsif($sota) {
            $orderString = "numlohi(DriveCartridgeAccess.\"DriveCartridgeAccessTimeUnmount\")";
        }
        else {
            $orderString = "numhilo(DriveCartridgeAccess.\"DriveCartridgeAccessTimeUnmount\")";
      }
    }

    if ($number) {
        $numberString = "$number";
        $flag = "q";
    }

    #
    # build report string
    #
    if ($longOutput) {
        $reportString .= $longReport;
    }
    elsif ($shortOutput) {
        $reportString .= $shortReport;
    }
    elsif ($ermmOutput) {
        $reportString .= "$className";
        $reportmode = "namevalue";
    }
    else {
        $reportString .= $defaultReport;
    }

    #
    # connect to TSLM server
    #
    Ermm::CLI::Main::connect();

    #
    # if number is not specified, print total number of objects.
    #
    if (!$flag) {
      my $sendString = Ermm::CLI::Main::composeMMPShow($Ermm::CLI::Conf::taskId,
                                                       $matchString,
                                                       $orderString,
                                                       $numberString,
                                                       $reportString,
                                                       "number");

      Ermm::CLI::Main::send($sendString);
      $response = Ermm::CLI::Main::getResponse();

      #
      # get total number of objects.
      #
      @values = Ermm::CLI::Main::getKeyValueResults($response);

      if ( ($#values < 0) || ($values[0][0] < 1) ) {
          Ermm::CLI::Main::printtext('EMM7009I');
          exit;
      }
    }
    my $records = $values[0][0];
    my $recread = 0;

    #
    # send command to server
    #
    Ermm::CLI::Main::sendMMPShow($Ermm::CLI::Conf::taskId,
                                 $matchString,
                                 $orderString,
                                 $numberString,
                                 $reportString,
                                 $reportmode);

    #
    # receive response from server
    #
    $response = Ermm::CLI::Main::getResponse();

    #
    # Begin loop
    #
    do {
        if ($ermmOutput) {
          #
          # -x option specified; print raw TSLM output
          #
          print $response,"\n";
        }
        else {
          #
          # start normal output processing
          #
          @values = Ermm::CLI::Main::getKeyValueResults($response);

          if ($#values < 0) {
              exit;
          }
          Ermm::CLI::Main::printResults($format, $delimiter, $header, $className,
                                      \@propNames, \@typeNames, \@values);
        }
        $recread += $Ermm::CLI::Conf::objlimit;
        if ($recread >= $records) {
            $flag = "q";
        }

        #
        # Get next part of response
        #
        if (!$flag) {
          Ermm::CLI::Main::nextMMPShowPart();
          #
          # receive response from server
          #
          $response = Ermm::CLI::Main::getResponse();

          #
          # Check if response has values
          #
            if ($response =~ m/text\s*\[.+\]/) {
              if ($Ermm::CLI::Conf::mode) {
                $flag = " ";
              }
              else {
                $flag = "";
              }
            }
            else {
              $flag = "q";
            }

          #
          # Prompt continue or quit
          #
            while ($flag ne "" && $flag ne "q") {
              Ermm::CLI::Main::printtext('CLIMSG_PROMT');
              $flag = <STDIN>;
              chomp $flag;
            }
        }

    }  while (!$flag)
}

#
# Call timelocal and print error message on invalid date string
#
sub timelocalcli {
    my ($sec,$min,$hour,$mday,$mon,$year) = @_;
    my $datetime;
    eval {$datetime = timelocal($sec,$min,$hour,$mday,$mon,$year)};
    if ($@) {
        Ermm::CLI::Main::printtext('EMM7125E', $_);
        exit;
    }
    return $datetime;
}

#
# Convert human readable date into db2 timestamp
# Since there are no perl modules for this in the
# "common perl" distribution we have to write our own.
#
# DB2:
#   TIMESTAMP ('2002-10-20-12.00.00.000000')
#   TIMESTAMP ('2002-10-20 12:00:00')  we will use this
#
# Humand: ISO 8601 subset
#   2007-02-01T11:53:01
#   2007/02/01T11:53:01
#   2007-02-01 11:53:01
#   2007/02/01 11:53:01
#   2007-02-01
#   11:53:01
#   11:40
#   2007-02-01 11:53
#   11d
#   11h
#   11m
#
sub Date2DB2 {
    ($_, $utc) = @_;
    my ($sec,$min,$hour,$mday,$mon,$year);

    if ($utc) {
        ($sec,$min,$hour,$mday,$mon,$year) = gmtime();
    }
    else {
        ($sec,$min,$hour,$mday,$mon,$year) = localtime();
    }

    #
    # react to different time strings and convert to UTC time.
    #
    if (/^(\d{1,2})[:](\d{1,2})(?:[:](\d{1,6}))?$/) {
        $hour = $1;
        $min  = $2;
        $sec  = $3;
        if (!$utc) {
            my $datetime = timelocalcli($sec,$min,$hour,$mday,$mon,$year);
            ($sec,$min,$hour,$mday,$mon,$year) = gmtime($datetime);
        }
    }
    elsif (/^(\d{4})[-](\d{1,2})[-](\d{1,2})$/) {
        $year = $1 - 1900;
        $mon  = $2 - 1;
        $mday = $3;
        $hour = 0;
        $min  = 0;
        $sec  = 0;
        if (!$utc) {
            my $datetime = timelocalcli($sec,$min,$hour,$mday,$mon,$year);
            ($sec,$min,$hour,$mday,$mon,$year) = gmtime($datetime);
        }
    }
    elsif (/^(\d{4})[-](\d{1,2})[-](\d{1,2})[T\.\ ](\d{1,2})[:](\d{1,2})(?:[:](\d{1,6}))?$/) {
        $year = $1 - 1900;
        $mon  = $2 - 1;
        $mday = $3;
        $hour = $4;
        $min  = $5;
        $sec  = $6;
        if (!$utc) {
            my $datetime = timelocalcli($sec,$min,$hour,$mday,$mon,$year);
            ($sec,$min,$hour,$mday,$mon,$year) = gmtime($datetime);
        }
    }
    elsif (/^[+-]?(\d+m)$/) {
        ($sec,$min,$hour,$mday,$mon,$year) = gmtime(time()-$1*60);
    }
    elsif (/^[+-]?(\d+h)$/) {
        ($sec,$min,$hour,$mday,$mon,$year) = gmtime(time()-$1*60*60);
    }
    elsif (/^[+-]?(\d+d)$/) {
        ($sec,$min,$hour,$mday,$mon,$year) = gmtime(time()-$1*60*60*24);
    }
    else {
        Ermm::CLI::Main::printtext('EMM7125E', $_);
        exit;
    }

    my $timestamp = sprintf ("%04d-%02d-%02d %02d:%02d:%02d", $year + 1900, $mon + 1, $mday, $hour, $min, $sec);
    return $timestamp;
}

1;
