#!/usr/bin/perl
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#-----------------------------------------------------------------------
#
# @version 1.3 - 12/01/24 01:49:03
#
# @brief
#
# Change activities:
#     - 2004-03-16: Initial version
#
#-----------------------------------------------------------------------

package Ermm::CLI::LsSlot;
use Getopt::Long;
use Term::ReadLine;
use Ermm::CLI::Conf;
use Ermm::CLI::Main;

$synopsis = "lsslot [-l | -s | -x | -v | -lib <name> | -bay <bay_name> | -sg <slot_group> | -empty | -pcl <pcl> | -hdr [on|off] | -help, -? | -match <c> | -order <c> | -number <c> | -fmt [xml|stanza|delim=c]] [<slotname>]";

#
# list slots
#
sub lsslot {
    my ($i, $longOutput, $shortOutput, $ermmOutput, $result);
    my ($response, $match, $order, $number, $library, $bay, $group, $empty, $pcl);
    my $help = 0;
    my $matchString = "";
    my $orderString = "";
    my $numberString = "";
    my $reportmode = "";
    my $reportString = "";
    my $flag = "";

    #
    # get defaults from configuration
    #
    my $format = $Ermm::CLI::Conf::format;
    my $delimiter = $Ermm::CLI::Conf::delimiter;
    my $header = $Ermm::CLI::Conf::header;

    #
    # the classname for xml output
    #
    my $className = "Slot";

    #
    # attributes listed by default
    #
    my $defaultReport =
        " Slot.\"SlotName\" Slot.\"LibraryName\" Slot.\"LMName\"".
        " Slot.\"BayName\" Slot.\"SlotGroupName\"".
        " Slot.\"SlotTypeName\" Slot.\"CartridgePCL\"";

    #
    # attributes listed if -l option given
    #
    my $longReport = $defaultReport.
        " Slot.\"SlotAccessible\" Slot.\"SlotOccupied\" Slot.\"SlotGroupName\"";

    #
    # attributes listed if -s option given
    #
    my $shortReport = "Slot.\"SlotName\"";

    #
    # attribute names on headline
    #
    my @propNames = ("Name", "Library", "LM",
                     "Bay", "Group",
                     "Type", "CartridgePCL",
                     "Accessible", "Occupied", "Slotgroup");
    #
    # attribute types
    #
    my @typeNames = ("string", "string", "string",
                     "string", "string",
                     "string", "string",
                     "string", "string", "string");

    #
    # process command line options
    #
    $result = GetOptions ("l"        => \$longOutput, # flag
                          "s"        => \$shortOutput,# flag
                          "x"        => \$ermmOutput, # flag
                          "v"        => \$Ermm::CLI::Conf::verbose, # flag
                          "fmt=s"    => \$format, # string
                          "help|?"   => \$help,   # flag
                          "hdr=s"    => \$header, # string
                          "bay=s"    => \$bay, # string
                          "lib=s"    => \$library, # string
                          "empty"    => \$empty, # flag
                          "sg=s"     => \$group, # string
                          "pcl=s"    => \$pcl, # string
                          "match=s"  => \$match, # string
                          "order=s"  => \$order, # string
                          "number=s" => \$number); # string

    #
    # check if we should print usage infos
    #
    if ($result != 1 || $help == 1) {
        Ermm::CLI::Main::printtext('_USAGE_LSSLOT', $synopsis);
        exit 1;
    }

    #
    # check header option
    #
    if ($header eq "on" || $header eq "yes") {
        $header = 1;
    }

    #
    # get delimiter from -fmt option
    #
    if ($format =~ /^delim=(.*)/) {
        $format = "delim";
        $delimiter = $1;
    }

    #
    # add drive names to match clause
    #
    foreach $i (1 .. $#ARGV) {
        $matchString .= "streq(Slot.\"SlotName\" \"$ARGV[$i]\")";
    }

    if ($#ARGV > 0) {
        $matchString = "or($matchString)";
    }

    #
    # add match clause if specified
    #
    if ($match) {
        $matchString = "and($matchString $match)";
    }

    #
    # add library restriction if specified
    #
    if ($library) {
        $matchString = "and($matchString streq(Slot.\"LibraryName\" ".
            "\"$library\"))";
    }
    #
    # add bay restriction if specified
    #
    if ($bay) {
        $matchString = "and($matchString streq(Slot.\"BayName\" ".
            "\"$bay\"))";
    }
    #
    # add slotgroup restriction if specified
    #
    if ($group) {
        $matchString = "and($matchString streq(Slot.\"SlotGroupName\" ".
            "\"$group\"))";
    }
    #
    # add pcl restriction if specified
    #
    if ($pcl) {
        $matchString = "and($matchString streq(Slot.\"CartridgePCL\" ".
            "\"$pcl\"))";
    }
    #
    # add empty restriction if specified
    #
    if ($empty) {
        $matchString = "and($matchString streq(Slot.\"SlotOccupied\" ".
            "\"N\"))";
    }

    if ($order) {
        $orderString = "$order";
    }
    else {
        $orderString = "strlohi(Slot.\"SlotName\")";
    }

    if ($number) {
        $numberString = "$number";
        $flag = "q";
    }

    #
    # build report string
    #
    if ($longOutput) {
        $reportString .= $longReport;
    }
    elsif ($shortOutput) {
        $reportString .= $shortReport;
    }
    elsif ($ermmOutput) {
        $reportString .= "$className";
        $reportmode = "namevalue";
    }
    else {
        $reportString .= $defaultReport;
    }

    #
    # connect to TSLM server
    #
    Ermm::CLI::Main::connect();

    #
    # if number is not specified, print total number of objects.
    #
    if (!$flag) {
      my $sendString = Ermm::CLI::Main::composeMMPShow($Ermm::CLI::Conf::taskId,
                                                       $matchString,
                                                       $orderString,
                                                       $numberString,
                                                       $reportString,
                                                       "number");

      Ermm::CLI::Main::send($sendString);
      $response = Ermm::CLI::Main::getResponse();

      #
      # get total number of objects.
      #
      @values = Ermm::CLI::Main::getKeyValueResults($response);

      if ( ($#values < 0) || ($values[0][0] < 1) ) {
          Ermm::CLI::Main::printtext('EMM7009I');
          exit;
      }
    }
    my $records = $values[0][0];
    my $recread = 0;

    #
    # send command to server
    #
    Ermm::CLI::Main::sendMMPShow($Ermm::CLI::Conf::taskId,
                                 $matchString,
                                 $orderString,
                                 $numberString,
                                 $reportString,
                                 $reportmode);

    #
    # receive response from server
    #
    $response = Ermm::CLI::Main::getResponse();

    #
    # Begin loop
    #
    do {
        if ($ermmOutput) {
          #
          # -x option specified; print raw TSLM output
          #
          print $response,"\n";
        }
        else {
          #
          # start normal output processing
          #
          @values = Ermm::CLI::Main::getKeyValueResults($response);

          if ($#values < 0) {
              exit;
          }
          Ermm::CLI::Main::printResults($format, $delimiter, $header, $className,
                                      \@propNames, \@typeNames, \@values);
        }
        $recread += $Ermm::CLI::Conf::objlimit;
        if ($recread >= $records) {
            $flag = "q";
        }

        #
        # Get next part of response
        #
        if (!$flag) {
          Ermm::CLI::Main::nextMMPShowPart();
          #
          # receive response from server
          #
          $response = Ermm::CLI::Main::getResponse();

          #
          # Check if response has values
          #
            if ($response =~ m/text\s*\[.+\]/) {
              if ($Ermm::CLI::Conf::mode) {
                $flag = " ";
              }
              else {
                $flag = "";
              }
            }
            else {
              $flag = "q";
            }

          #
          # Prompt continue or quit
          #
            while ($flag ne "" && $flag ne "q") {
              Ermm::CLI::Main::printtext('CLIMSG_PROMT');
              $flag = <STDIN>;
              chomp $flag;
            }
        }

    }  while (!$flag)
}

1;
