#!/usr/bin/perl -w
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#-----------------------------------------------------------------------
#
# @version 1.3 - 12/01/24 01:49:13
#
# @brief
#
# Change activities:
#     - 2007-04-18: Initial version
#
#-----------------------------------------------------------------------

package Ermm::CLI::LsVu;
use Getopt::Long;
use Term::ReadLine;
use Ermm::CLI::Conf;
use Ermm::CLI::Main;

$synopsis = "lsvu [-l | -v | -hdr [on|off] | -help, -? | -fmt [xml|stanza|delim=c]]";

#
# list value units
#
sub lsvu {
    my ($longOutput, $shortOutput, $result);
    my $response;
    my $valueUnits = 0;
    my $help = 0;
    my $flag = "";

    #
    # get defaults from configuration
    #
    my $format = $Ermm::CLI::Conf::format;
    my $delimiter = $Ermm::CLI::Conf::delimiter;
    my $header = $Ermm::CLI::Conf::header;

    #
    # the classname for xml output
    #
    my $className = "ValueUnit";

    #
    # attribute names on headline
    #
    my @propNames = ("ValueUnits", "Drives");

    #
    # attribute types
    #
    my @typeNames = ("int", "int");

    #
    # process command line options
    #
    $result = GetOptions ("l"        => \$longOutput, # flag
                          "s"        => \$shortOutput,# flag
                          "v"        => \$Ermm::CLI::Conf::verbose, # flag
                          "fmt=s"    => \$format, # string
                          "help|?"   => \$help,   # flag
                          "hdr=s"    => \$header); # string

    #
    # check if we should print usage infos
    #
    if ($result != 1 || $help == 1) {
        Ermm::CLI::Main::printtext('_USAGE_LSVU', $synopsis);
        exit 1;
    }

    #
    # check header option
    #
    if ($header eq "on" || $header eq "yes") {
        $header = 1;
    }

    #
    # get delimiter from -fmt option
    #
    if ($format =~ /^delim=(.*)/) {
        $format = "delim";
        $delimiter = $1;
    }

    #
    # Build the fixed send string which gets the total number of drives
    # managed by TSLM. This is the base for calculation of the value units.
    #
    $sendString = "show task[\"$Ermm::CLI::Conf::taskId\"] report[Drive.\"DriveName\"] reportMode[number];";

    #
    # connect to TSLM server and get the response
    #
    Ermm::CLI::Main::connect();
    Ermm::CLI::Main::send($sendString);
    $response = Ermm::CLI::Main::getResponse();

    #
    # start normal output processing
    #
    @values = Ermm::CLI::Main::getKeyValueResults($response);

    my $numberOfDrives;
    if ( ($#values < 0) || ($values[0][0] < 1) ) {
        # No drives found (do not display message "no records found")
        $numberOfDrives = 0;
    }
    else {
        $numberOfDrives = $values[0][0];
    }

    #
    # calculate value units based on the number of drives
    #
    $valueUnits = calculateValueUnits($numberOfDrives);

    #
    # Display the result
    #
    # Note: This command differs to others as the MMP show command
    #       only includes a reportMode "number". In order to use the
    #       same display functions the @values array needs to be
    #       manipulated accordingly.
    #

    if ($longOutput) {
	$values[0][0] = $valueUnits;
	$values[0][1] = $numberOfDrives;
    }
    else {
	$values[0][0] = $valueUnits;
    }

    Ermm::CLI::Main::printResults($format, $delimiter, $header, $className,
                                \@propNames, \@typeNames, \@values);

}

#
# Calculate the total value units
#
sub calculateValueUnits {

    my $numberOfDrives = shift;
    my $totalValueUnit = 0;
    my $i;
    my $currentUsageLevel = 1;

    #
    # Initialize the LevelValueUnits Hash
    #   key   =  cumulative usage level
    #   value =  value units
    #
    my %LevelValueUnits = ("1" => "10",
                           "2" => "9",
                           "3" => "8",
                           "4" => "7",
                           "5" => "6",
                           "6" => "5",
                           "7" => "4",
                           "8" => "3");

    #
    # Initialize the LevelDrives Hash
    #   key   =  cumulative usage level
    #   value =  upper limit of drives
    #
    my %LevelValueDrives = ("1" => "20",
                            "2" => "40",
                            "3" => "60",
                            "4" => "80",
                            "5" => "100",
                            "6" => "140",
                            "7" => "180",
                            "8" => "999999");

    if ($numberOfDrives == 0) {
        return 0;
    }
    else {
        for ($i=1; $i<=$numberOfDrives; $i++) {
            if ($i > $LevelValueDrives{$currentUsageLevel}) {
		$currentUsageLevel++;
            }
	    $TotalValueUnit += $LevelValueUnits{$currentUsageLevel};
	}
    }

    return $TotalValueUnit;
}

1;
