#!/usr/bin/perl
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#-----------------------------------------------------------------------
#
# @version 1.3 - 12/01/24 02:06:09
#
# @brief
#
#   Module with TSLM CLI commonly used functions
#
#-----------------------------------------------------------------------

package Ermm::CLI::Main;

require 5.002;
use Socket qw(:DEFAULT :crlf);
use constant BASE64 => do{ eval{ require MIME::Base64 }};
use constant PING   => do{ eval{ require Net::Ping }};
use constant IPV6   => do{ eval{ require Socket::Class }};
use Ermm::CLI::Conf;
use Time::Local;
use Ermm::CLI::CLIVersion;
use nls::cli_L10N;

$VERSION = "1.1.0";

#
#IPv6 Socket object for Client
my $clientSocket;

# Special flag to indicate whether a reordering of the IP list
# in the configuration file should be surpressed. This will be set
# in case the -s option of ermmtool is specified.
$noConfFileReorder = 0;

#
# Connect to Media Manager using the credentials
# specified in Ermm::CLI::Conf.pm.
#
# In case there are multiple IP addresses iterate through the list
# in order to find a running Media Manager.
#
sub connect {
    my($iaddr, $paddr, $proto, $line);
    my @ipAddresses = ();
    my $numberOfIps = 0;
    my $index       = 0;
    my $reorder     = 0;
    my $foundIP     = 0;
    my $IPv4_Flag   = 0;
    my $IPv6_Flag   = 0;
    my $IPPort      = "";
    my $IPAddress;
    my $ret;
    my $sock;

    # Check for IPv6 configuration
    if ($Ermm::CLI::Conf::ipv6support > 0 && !IPV6) {
       Ermm::CLI::Main::printtext('EMM7023E');
       exit;
    }

    @ipAddresses = split(/,/,$Ermm::CLI::Conf::host);
    $numberOfIps = $#ipAddresses+1;

    #
    # Check for mixed IPv4/IPv6 addresses in a list of IP-addresses
    #
    foreach my $currentIp (@ipAddresses) {

       # Parse the IP-address
       ($IPAddress, $IPPort) = parsingIpAddress($currentIp);

       # Verify it is a IPv4 Adresse (first character is numerisch and no ':')
       if ((substr($IPAddress,0,1) =~ /[1-9]/) && (index($IPAddress,':') == -1)) {
          $IPv4_Flag = 1;
       }
       else {
         # Verify it is a IPv6 Adresse (IP address contains a ':')
         if (index($IPAddress,':') > -1) {
            $IPv6_Flag = 1;
         }
       }
    }
    # Error message if we found IPv4 and IPv6 address in the IP-List
    if (($IPv4_Flag) && ($IPv6_Flag)) {
       Ermm::CLI::Main::printtext('EMM7026E');
       exit 2;
    }

    #
    # Start connection processing
    #
    foreach my $currentIp (@ipAddresses) {
      $index++;

      # Parse the IP-address
      ($IPAddress, $IPPort) = parsingIpAddress($currentIp, $Ermm::CLI::Conf::port);

      #
      # IPv6 connection
      #
      if ($Ermm::CLI::Conf::ipv6support > 0 && IPV6) {

          # Verify hostname/ip address is valid for IPv6
          $sock = Socket::Class->new();
          $sock->getaddrinfo($IPAddress, $IPPort, 'inet6', UNDEF, 'STREAM');

          if ($sock->is_error ||
              $sock->error)   {
             Ermm::CLI::Main::printtext('EMM7024E', $IPAddress);
             $sock->free;
             exit 2;
          }

          $sock->free;

          # create a new socket and connect to the IPv6 server
          $clientSocket = Socket::Class->new(
                              'remote_addr' => $IPAddress,
                              'remote_port' => $IPPort,
                              'domain'      => "inet6",
          );

          # An error occured
          if (Socket::Class->is_error ||
              Socket::Class->error)   {
             if ($numberOfIps == 1) {
                Ermm::CLI::Main::printtext('EMM7013E', Socket::Class->error);
                exit 2;
             }
             elsif ($numberOfIps > 1) {
                Ermm::CLI::Main::printtext('EMM7013E', Socket::Class->error);
                if ($index < $numberOfIps){
	           Ermm::CLI::Main::printtext('EMM7016I');
                   $reorder=1;
                   next;
                }
                else {
                   exit 2;
                }

             }
          }
      }
      #
      # IPv4 connection
      #
      else {

         # IPv6 Address specified but IPv6 support is enabled
         if (index($IPAddress,':') < rindex($IPAddress,':')) {
            Ermm::CLI::Main::printtext('EMM7025E', $IPAddress);
            exit 2;
         }

         # Check if a given address is reachable on the network.
         # The check is only done if there is a timeout value greater than 0.
         # This mechanism is faster than waiting for the socket operation timeout
         # (or modifying any socket options).
         if (PING && ($Ermm::CLI::Conf::timeout != 0)) {
            my $pingHost = Net::Ping->new();

            # Remark: $pingHost->port_number($IPPort) is only availbale with version 2.33 (July 2007)
            # Since this is not the default version we don't use this option (yet)
            if (!($pingHost->ping($IPAddress, $Ermm::CLI::Conf::timeout))) {
               Ermm::CLI::Main::printtext('EMM7015E', $IPAddress, $Ermm::CLI::Conf::timeout);
               $pingHost->close();

               if ($numberOfIps == 1) {
                  exit 2;
               }
               elsif ($index < $numberOfIps) {
	          Ermm::CLI::Main::printtext('EMM7016I');
                  $reorder=1;
                  next;
               }
               else {
                  exit 2;
               }
            }
            $pingHost->close();
         }

         if ($numberOfIps == 1) {
            $iaddr = inet_aton($IPAddress) or
	       die Ermm::CLI::Main::gettext('EMM7005E', $IPAddress);
         }
         else {
            $iaddr = inet_aton($IPAddress) or
               warn Ermm::CLI::Main::gettext('EMM7005E', $IPAddress);
         }
         $paddr = sockaddr_in($IPPort, $iaddr);
         $proto = getprotobyname('tcp');

         #
         # create a socket
         #
         if ($numberOfIps == 1) {
            socket(SOCK, PF_INET, SOCK_STREAM, $proto) or
               die Ermm::CLI::Main::gettext('EMM7013E', $!);
         }
         else{
            socket(SOCK, PF_INET, SOCK_STREAM, $proto) or
              warn Ermm::CLI::Main::gettext('EMM7013E', $!);
         }

         setsockopt(SOCK, SOL_SOCKET, SO_KEEPALIVE, 1);

         #
         # and try to connect to the TSLM MediaManager
         #
         if ($numberOfIps == 1) {
            connect(SOCK, $paddr) or
               die Ermm::CLI::Main::gettext('EMM7002E', $IPAddress, $IPPort, $!);
         }
         else {
            $ret = connect(SOCK, $paddr);
            if (!$ret) {
	       Ermm::CLI::Main::printtext('EMM7002E', $IPAddress, $IPPort, $!);
               if ($index < $numberOfIps){
	          Ermm::CLI::Main::printtext('EMM7016I');
                  $reorder=1;
                  next;
               }
               else {
                  exit 1;
               }
            }
         }
      }

      if ($Ermm::CLI::Conf::authMethod =~ m/pwenc/i) {
        if (BASE64) {
          $Ermm::CLI::Conf::password =
            getEncryptedPassword($Ermm::CLI::Conf::client,
                                 $Ermm::CLI::Conf::instance);
        }
        else {
	    die Ermm::CLI::Main::gettext('EMM7004E');
        }
      }

      #
      # start session and authenticate ourself
      #
      if ($Ermm::CLI::Conf::verbose > 0) {
        Ermm::CLI::Main::printtext('CLIVMSG1001I');
      }

      if ($Ermm::CLI::Conf::debug){ 
	Ermm::CLI::Main::printtext('CLIVMSG1010I', $Ermm::CLI::Conf::client, $Ermm::CLI::Conf::instance, $Ermm::CLI::Conf::password);
      }

      my $hello_string = "";
      if ($Ermm::CLI::CLIVersion::CLIVERSION eq "CLIVERSIONSTRING") {
        $hello_string = "hello client[\"$Ermm::CLI::Conf::client\"] ".
	    " instance[\"$Ermm::CLI::Conf::instance\"] ".
	    " password[\"$Ermm::CLI::Conf::password\"] ".
            " language[\"mmp\"];\n";
        }
      else {
	$hello_string = "hello client[\"$Ermm::CLI::Conf::client\"] ".
	    " instance[\"$Ermm::CLI::Conf::instance\"] ".
	    " password[\"$Ermm::CLI::Conf::password\"] ".
	    " language[\"mmp\"] clientversion [\"$Ermm::CLI::CLIVersion::CLIVERSION\"];\n";
      }
      Ermm::CLI::Main::send($hello_string);

      #
      # recieving the answer from MM
      #
      $line = readSocket();
      if (!defined $line) {
        Ermm::CLI::Main::printtext('EMM7003E');
        closeSocket();
        if ($index < $numberOfIps){
          Ermm::CLI::Main::printtext('EMM7016I');
          $reorder=1;
          next;
        }
        else {
          exit 1;
        }
      }

      if (!defined $line || $line =~ /unwelcome/) {
	Ermm::CLI::Main::printtext('EMM7014E', $line);
        closeSocket();
        if ($index < $numberOfIps){
          Ermm::CLI::Main::printtext('EMM7016I');
          $reorder=1;
          next;
        }
        else {
          exit 1;
        }
      }
      else {
        # ensure we stop loop iterarion if we have a valid connection
        $foundIP=1;
      }

      if ($foundIP) { last };

    } # end of foreach

    if ($Ermm::CLI::Conf::verbose > 0) {
      Ermm::CLI::Main::printtext('CLIVMSG1002I');
    }

    # Only reorder/rewrite the IP list in the configuration file if there is a list of
    # IP address and an address other than the first was reachable on the network.
    # If a list was provided with the -s option we surpress reordering the configuration file.
    if (($numberOfIps>1) && $reorder && $foundIP && !$noConfFileReorder) {
       reorderIPs($IPAddress,$IPPort);
    }

}

#
# send a command string to the TSLM server
#
sub send {
    my $line = shift;

    if ($Ermm::CLI::Conf::verbose > 0) {
        Ermm::CLI::Main::printtext('CLIVMSG1003I', $line);
    }

    #
    # check if CLI started on z/OS platform
    # in this case EBCDIC-ASCII convertion is necessary
    #
    if ($^O =~ /.*os390.*/i) {
        $line = EBCDICtoASCII($line);
    }

    if ($Ermm::CLI::Conf::ipv6support > 0 && IPV6) {
        $clientSocket->send($line) or die $clientSocket->error;
    }
    else {
        send SOCK, $line, 0;
    }
}

#
# send MMP show command to the TSLM server
#
sub sendMMPShow ($$$$$$) {
  $task = shift;
  $match = shift;
  $order = shift;
  $number = shift;
  $report = shift;
  $reportmode = shift;

  my $sendString = "";
  @interval = (1, $Ermm::CLI::Conf::objlimit);

  if (!$number) {
    $number = "$interval[0]..$interval[1]";
  }

  $sendString = composeMMPShow($task, $match, $order, $number, $report, $reportmode);
  Ermm::CLI::Main::send($sendString);
}

sub nextMMPShowPart {
  $interval[0] = $interval[1] + 1;
  $interval[1] += $Ermm::CLI::Conf::objlimit;

  $number = "$interval[0]..$interval[1]";
  my $sendString = composeMMPShow($task, $match, $order, $number, $report, $reportmode);
  Ermm::CLI::Main::send($sendString);
}

sub composeMMPShow ($$$$$$) {
  my $t = shift;
  my $m = shift;
  my $o = shift;
  my $n = shift;
  my $r = shift;
  my $rm = shift;

  my $mmpshow = "show task[\"$t\"]";
  if ($m) {
    $mmpshow .= " match[$m]";
  }
  if ($o) {
    $mmpshow .= " order[$o]";
  }
  if ($n) {
    $mmpshow .= " number[$n]";
  }
  if ($r) {
    $mmpshow .= " report[$r]";
  }
  if ($rm) {
    $mmpshow .= " reportmode[$rm]";
  }
  $mmpshow .= ";";

  return $mmpshow;
}

#
# disconnect from the TSLM server
#
sub disconnect {
    my $msg = "goodbye task[\"cliexit\"];\n";
    Ermm::CLI::Main::send($msg);
    getResponse(SOCK);

    if ($Ermm::CLI::Conf::verbose > 0) {
        Ermm::CLI::Main::printtext('CLIVMSG1004I');
    }
    closeSocket();
}

#
# waiting only for accept response
#
sub getAcceptedResponse {
    my $line = '';
    my $error = 0;

    do {
        $line = readSocket();
        if (defined($line) && $line) {
            # remove leading and trailing space chars
            $line =~ s/^\s+//;
            $line =~ s/\s+$//;
        }
        else {
            Ermm::CLI::Main::printtext('EMM7121E', $!);
            exit 1;
        }
    } while (length($line) == 0);

    if ($Ermm::CLI::Conf::verbose > 0) {
        Ermm::CLI::Main::printtext('CLIVMSG1005I', $line);
    }

    if ($line =~ /^\s*response\s+unacceptable.*/) {
        if ($Ermm::CLI::Conf::verbose > 0) {
            Ermm::CLI::Main::printtext('CLIVMSG1006I', $line);
        }
        Ermm::CLI::Main::printtext('EMM7118E');
        exit 1;
    }
}

#
# waiting for accept and success responses
#
sub getResponse {
    my $dontExitOnError = shift;
    my $line = '';
    my $error = 0;

    do {
        $line = readSocket();
        if (defined($line) && $line) {
            # remove leading and trailing space chars
            $line =~ s/^\s+//;
            $line =~ s/\s+$//;
        }
        else {
            Ermm::CLI::Main::printtext('EMM7121E', $!);
            exit 1;
        }
    } while (length($line) == 0);

    if ($Ermm::CLI::Conf::verbose > 0) {
        Ermm::CLI::Main::printtext('CLIVMSG1005I', $line);
    }

    if ($line =~ /^\s*response\s+unacceptable.*/) {
        if ($Ermm::CLI::Conf::verbose > 0) {
            Ermm::CLI::Main::printtext('CLIVMSG1006I', $line);
        }
        Ermm::CLI::Main::printtext('EMM7118E');
        exit 1;
    }

    do {
        $line = readSocket();
        if (defined($line) && $line) {
            # remove leading and trailing space chars
            $line =~ s/^\s+//;
            $line =~ s/\s+$//;
        }
        else {
            Ermm::CLI::Main::printtext('EMM7121E', $!);
            exit 1;
        }
    } while (length($line) == 0);

    if ($Ermm::CLI::Conf::verbose > 0) {
        Ermm::CLI::Main::printtext('CLIVMSG1005I', $line);
    }

    #
    # remove trailing ;
    #
    $line =~ s/;$//;

    $status = getResponseStatus($line);

    if (defined $dontExitOnError && $dontExitOnError) {
        return $line;
    }

    if (lc($status) eq "unacceptable") {
        Ermm::CLI::Main::printtext('EMM7118E');
        exit 1;
    }
    elsif (lc($status) eq "cancelled") {
        Ermm::CLI::Main::printtext('EMM7119E');
        exit 1;
    }
    elsif (lc($status) eq "error") {
        my $errorCode = getResponseErrorCode($line);
        my $info = getResponseMessage($line);
        if ($info eq "") {
            Ermm::CLI::Main::printtext('EMM7120E', $errorCode);
        }
        else {
            print "$info\n";
        }
        exit 1;
    }

    return $line;
}

#
# waiting for notification
#
sub getNotification {
    my $dontExitOnError = shift;
    my $line = '';
    my $error = 0;

    do {
        $line = readSocket();
        if (defined($line) && $line) {
            # remove leading and trailing space chars
            $line =~ s/^\s+//;
            $line =~ s/\s+$//;
        }
        else {
            Ermm::CLI::Main::printtext('EMM7126E', $!);
            exit 1;
        }
    } while (length($line) == 0);

    if ($Ermm::CLI::Conf::verbose > 0) {
        Ermm::CLI::Main::printtext('CLIVMSG1007I', $line);
    }

    #
    # remove trailing ;
    #
    $line =~ s/;$//;

    $status = getResponseStatus($line);

    if (defined $dontExitOnError && $dontExitOnError) {
        return $line;
    }

    if (lc($status) eq "unacceptable") {
        Ermm::CLI::Main::printtext('EMM7118E');
        exit 1;
    }
    elsif (lc($status) eq "cancelled") {
        Ermm::CLI::Main::printtext('EMM7119E');
        exit 1;
    }
    elsif (lc($status) eq "error") {
        my $errorCode = getResponseErrorCode($line);
        my $info = getResponseMessage($line);
        print "$info\n";
        exit 1;
    }

    return $line;
}

sub closeSocket {

    if ($Ermm::CLI::Conf::ipv6support > 0 && IPV6) {
       $clientSocket->free();
    }
    else {
       close(SOCK);
    }
}

sub getTaskId {
    my $line = shift;

    $line =~ /^\s*response\s+task\s*\[\"([^\"]*)\"\].*/;
    return $1;
}

sub getResponseStatus {
    my $line = shift;

    if ($line =~ /^\s*response\s+unacceptable.*/) {
        return "unacceptable";
    }
    $line =~ /^\s*response\s+task\s*\[\"([^\"]*)\"\]\s*([a-zA-Z]+).*/;
    return $2;
}

sub getResponseErrorCode {
    my $line = shift;

    $line =~ /^\s*response\s+task\s*\[\s*\"([^\"]*)\"\s*\]\s*error\s*\[\s*\"([^\"]*)\"\s*\"([^\"]*)\"\s*\].*/;
    return "$3";
}

sub getResponseMessage {
    my $line = shift;

    $line =~ /.*loctext\s*\[\s*\"([^\"]*)\"\s*\"([^\"]*)\"\s*\].*/;
    return "$1";
}

sub getKeyValueResults {
    my $line = shift;
    my @values;
    my $i = 0;
    my @objs = (split /\btext\s*\[/, $line);

    for ($i = 1; $i <= $#objs; $i++) {
        $_ = $objs[$i];
        # remove trailing space and bracket chars
        s/\]\s*$//;
        # remove leading and trailing quotes and outside blanks
        s/^\s*\"//;
        s/\"\s*$//;
        # split results at " "
        push @{$values[$i-1]}, (split /\"\s+\"/) ;
    }
    return @values;
}

sub printResults {
    my $format = shift;
    my $delimiter = shift;
    my $header = shift;
    my $className = shift;
    my $propNames = shift;
    my $typeNames = shift;
    my $values = shift;

    if ($format eq "xml") {
        printXMLResults($className, $propNames, $typeNames, $values);
    }
    elsif ($format eq "stanza") {
        printStanzaResults($propNames, $typeNames, $values);
    }
    else {
        printCharDelimitResults($delimiter, $header,
                                $propNames, $typeNames, $values);
    }
}

sub printXMLResults {
    my $className = shift;
    my $props = shift;
    my $types = shift;
    my $vals = shift;
    my @propNames = @$props;
    my @typeNames = @$types;
    my @values = @$vals;
    my $o;
    my $i;

    print "<IRETURNVALUE>\n";
    for $o (0 .. $#values) {
        print "<INSTANCE CLASSNAME=\"$className\">\n";
        for $i (0 .. $#{$values[$o]}) {
            print "<PROPERTY NAME=\"$propNames[$i]\" TYPE=\"$typeNames[$i]\">";
            if ($typeNames[$i] eq "dateTime") {
                print "<VALUE>".toIsoStringLocal($values[$o][$i]).
                    "</VALUE></PROPERTY>\n";
            }
            elsif ($typeNames[$i] eq "dateTimeUTC") {
                print "<VALUE>".toIsoStringUTC($values[$o][$i]).
                    "</VALUE></PROPERTY>\n";
            }
            else {
                print "<VALUE>$values[$o][$i]</VALUE></PROPERTY>\n";
            }
        }
        print "</INSTANCE>\n";
    }
    print "</IRETURNVALUE>\n";
}

sub printCharDelimitResults {
    my $delimiter = shift;
    my $printHeader = shift;
    my $props = shift;
    my $types = shift;
    my $vals = shift;
    my @propNames = @$props;
    my @typeNames = @$types;
    my @values = @$vals;
    my $o;
    my $i;
    my $l;
    my $headString = "";
    my @colwidth;
    my $totalwidth = 0;
    my $numcols = $#{$values[0]};

    if ($numcols == -1) {
        $printHeader = "off";
    }

    if ($printHeader ne "1" && $printHeader ne "0") {
        if (lc($printHeader) eq "off") {
            $printHeader = 0;
        }
        else {
            Ermm::CLI::Main::printtext('EMM7136W');
        }
    }
    elsif ($printHeader eq "0") {
        $printHeader = 1;
    }

    if ($printHeader) {
        #
        # get max number of columns to print
        #
        for $o (0 .. $#values) {
            if ($numcols < $#{$values[$o]}) {
                $numcols = $#{$values[$o]};
            }
        }
        #
        # get max size of every column
        #
        for ($i = 0; $i <= $numcols; $i++) {
            push @colwidth, length($propNames[$i]);
        }
        for $o (0 .. $#values) {
            for $i (0 .. $#{$values[$o]}) {
                if ($typeNames[$i] eq "dateTime") {
                    $l = 19;
                }
                else {
                    $l = length($values[$o][$i]);
                }
                if ($l > $colwidth[$i]) {
                    $colwidth[$i] = $l;
                }
            }
        }
        #
        # compute total width of header
        #
        for $i (0 .. $#colwidth) {
          $totalwidth += $colwidth[$i];
          if ($i < $#colwidth) {
            $totalwidth += length($delimiter);
          }
        }
    }
    if ($printHeader) {
        for ($i = 0; $i <= $numcols; $i++) {
            if ($i > 0) {
                $headString .= $delimiter;
            }
            $headString .= $propNames[$i];
            #
            # fill up column with blanks in case it's needed
            #
            my $ldif = $colwidth[$i] - length($propNames[$i]);
            if ($ldif > 0) {
                $headString .= " " x $ldif;
            }
        }
        print "$headString\n";
        print "=" x $totalwidth;
        print "\n";
    }
    for $o (0 .. $#values) {
        for $i (0 .. $#{$values[$o]}) {
            if ($i > 0) {
                print $delimiter;
            }
            if ($typeNames[$i] eq "dateTime") {
                print toIsoStringLocal($values[$o][$i]);
                if ($printHeader) {
                    my $ldif = $colwidth[$i] - 19;
                    if ($ldif > 0) {
                        print " " x $ldif;
                    }
                }
            }
            elsif ($typeNames[$i] eq "dateTimeUTC") {
                print toIsoStringUTC($values[$o][$i]);
                if ($printHeader) {
                    my $ldif = $colwidth[$i] - 19;
                    if ($ldif > 0) {
                        print " " x $ldif;
                    }
                }
            }
            else {
                print $values[$o][$i];
                if ($printHeader) {
                    my $ldif = $colwidth[$i] - length($values[$o][$i]);
                    if ($ldif > 0) {
                        print " " x $ldif;
                    }
                }
            }
        }
        print "\n";
    }
}

sub printStanzaResults {
    my $props = shift;
    my $types = shift;
    my $vals = shift;
    my @propNames = @$props;
    my @typeNames = @$types;
    my @values = @$vals;
    my $o;
    my $i;
    my $pcolwidth = 0;
    my $numcols = $#{$values[0]};

    #
    # get max size of every column
    #
    for ($i = 0; $i <= $numcols; $i++) {
        if ($pcolwidth < length($propNames[$i])) {
            $pcolwidth = length($propNames[$i]) + 1;
        }
    }
    for $o (0 .. $#values) {
        for $i (0 .. $#{$values[$o]}) {
            my $ldif = $pcolwidth - length($propNames[$i]);
            print $propNames[$i];
            if ($ldif > 0) {
                print " " x $ldif;
            }
            if ($typeNames[$i] eq "dateTime") {
                print toIsoStringLocal($values[$o][$i])."\n";
            }
            elsif ($typeNames[$i] eq "dateTimeUTC") {
                print toIsoStringUTC($values[$o][$i])."\n";
            }
            else {
                print "$values[$o][$i]\n";
            }
        }
        print "\n";
    }
}

sub toIsoStringLocal {
    my ($t, $utc) = @_;
    $t =~ /(\d+)\s(\d+)\s(\d+)\s(\d+)\s(\d+)\s(\d+)\s(\d+)/;
    my ($sec,$min,$hour,$mday,$mon,$year) = ($6, $5, $4, $3, $2, $1);

    if ($t =~ /^1900 01 01 00 00 00/) {
        return sprintf("1900-01-01T00:00:00");
    }
    my $datetime = timegm($sec,$min,$hour,$mday,$mon-1,$year-1900);
    ($sec,$min,$hour,$mday,$mon,$year) = localtime($datetime);

    return sprintf("%4d-%02d-%02dT%02d:%02d:%02d", $year+1900,$mon+1,$mday,$hour,$min,$sec);
}

sub toIsoStringUTC {
    my ($t, $utc) = @_;
    $t =~ /(\d+)\s(\d+)\s(\d+)\s(\d+)\s(\d+)\s(\d+)\s(\d+)/;
    my ($sec,$min,$hour,$mday,$mon,$year) = ($6, $5, $4, $3, $2, $1);

    return sprintf("%4d-%02d-%02dT%02d:%02d:%02d", $year,$mon,$mday,$hour,$min,$sec);
}

sub encrypt($) {
    my $s = shift;
    my $d = "";

    for (my $i = 0; $i < length($s); $i++) {
        $d .= chr(ord(substr($s, $i, 1)) ^ 0xA5);
    }
    return $d;
}

sub getEncryptedPassword() {
    my $app = shift;
    my $ai = shift;
    my $pw = "";
    my ($fapp, $fai, $fpw, $rest, $line);

    if (open(PWFILE, "/opt/IBM/ermm/conf/ermmsecurity.conf")) {
        while ($line = <PWFILE>) {
            ($fapp, $fai, $fpw, $rest) = split(":", $line);
            if ($fapp eq $app && $fai eq $ai) {
                return MIME::Base64::encode_base64(encrypt($fpw), "");
            }
        }
    }
    else {
        Ermm::CLI::Main::printtext('EMM7127E');
        exit 1;
    }
    return $pw;
}


#
# read the stream from the socket with the use of ASCII-EBCDIC converter, if needed
#
sub readSocket() {
       my $line        = '';
       my $inputBuffer = '';
       my $bufSize     = 1;

       #
       # IPv6 connection
       #
       if ($Ermm::CLI::Conf::ipv6support > 0 && IPV6) {
          until (index($line,';') > -1 || $clientSocket->is_error) {
             $inputBuffer =  '';
             if ($clientSocket->is_readable(100)) {
                $clientSocket->read($inputBuffer,$bufSize);
                $line .= $inputBuffer;
             }
          }

          if ($clientSocket->is_error) {
             Ermm::CLI::Main::printtext('EMM7002E', $clientSocket->remote_addr, $clientSocket->remote_port, $!);
             exit 1;
          }
       }
       #
       # IPv4 connection
       #
       else {
          # Special handling for z/OS
          # You have keep this lines together or it doesn't work
          if ($^O =~ /.*os390.*/i) {
             local($/) = LF;
             $line = <SOCK>;
          }
          else {
             $line = <SOCK>;
          }
       }

       # For z/OS translate output to EBCDIC
       if ($^O =~ /.*os390.*/i) {
          $line = ASCIItoEBCDIC($line);
       }
       return $line;
}

#
# Reorder the list of IP addresses in the configuration file(s).
# The last reachable IP will be put in front of the list.
# In case there is a problem reading or writing the file
# in this function we do not exit the program.
#
sub reorderIPs() {
    my $activeIp    = shift;
    my $activePort  = shift;

    my $index=0;
    my $update = 0;
    my $conffile;
    my @ipAddresses = ();
    my @conffiles = ();
    my @lines = ();
    my $IPAddress;
    my $IPPort;


    @ipAddresses = split(/,/,$Ermm::CLI::Conf::host);
    foreach my $currentIp (@ipAddresses) {

        $IPAddress = "";
        $IPPort = "";

        #Extract the port information.
        ($IPAddress, $IPPort) = parsingIpAddress($currentIp);

        if ($IPPort eq "" || (!defined $IPPort)) {

            if (($IPAddress eq $activeIp) && ($Ermm::CLI::Conf::port eq $activePort)) { last };
        }
        else {
            $IPPort =~ s/^\s*//;
            $IPPort =~ s/\s*$//;
            if (($IPAddress eq $activeIp) && ($IPPort eq $activePort)) { last };
        }

        $index++;

    }

    # switch the IP addresses in the list
    my $switchIp = $ipAddresses[0];
    if ($IPPort eq "" || (!defined $IPPort)) {
        $ipAddresses[0] = $activeIp;
    }
    else {
        # IPv6 IP-address in brackets
        if ($Ermm::CLI::Conf::ipv6support > 0) {
           $ipAddresses[0] = "[".$activeIp."]:".$activePort;
        }
        else {
           $ipAddresses[0] = $activeIp.":".$activePort;
        }
    }
    $ipAddresses[$index] = $switchIp;

    # Update the host information
    $Ermm::CLI::Conf::host = join (',',@ipAddresses);

    # Reorder the list of IP addresses in the configuration file(s)
    # (if the file exists).
    # Configuration file in /etc is used first.
    # Configuration file in home directory is used second.
    if ( -e "/etc/".$Ermm::CLI::Conf::configname) {
        push(@conffiles, "/etc/".$Ermm::CLI::Conf::configname);
    }
    if ( -e $ENV{HOME}."/".".".$Ermm::CLI::Conf::configname) {
        push(@conffiles, $ENV{HOME}."/".".".$Ermm::CLI::Conf::configname);
    }

    if (!($#conffiles >= 0)) {
        Ermm::CLI::Main::printtext('EMM7017I');
    }
    else {
        foreach $conffile (@conffiles) {
            open(CONFFILE, $conffile) or
                warn Ermm::CLI::Main::gettext('EMM7018I', $conffile);

            @lines = <CONFFILE>;

            close(CONFFILE) or
                warn Ermm::CLI::Main::gettext('EMM7019I', $conffile);

            # Create a "new" configuration file with the modified host list
            open(CONFFILE, ">$conffile") or
                warn Ermm::CLI::Main::gettext('EMM7018I', $conffile);

            foreach my $line (@lines) {
                # Check if the string in the config file is of the format
                # VARIABLE=VALUE or VARIABLE="VALUE".
                if($line =~ m/^\s*(\w+)\s*=\s*((?:\".*?\")|(?:[^\"]*?))\s*$/g) {
                    $var = $1;
                    $val = $2;

                    if($var =~ m/^\s*HOST\s*$/i) {
                        # Update the list
                        $line = $var."=".$Ermm::CLI::Conf::host."\n";
                        $update = 1;
                    }
                }

                # Write changed information back to file
                print CONFFILE $line;
            }

            close(CONFFILE)or
                warn Ermm::CLI::Main::gettext('EMM7019I', $conffile);

        }
    }

    if ($update) {
        Ermm::CLI::Main::printtext('EMM7020I', $ipAddresses[0]);
    }
    else {
        Ermm::CLI::Main::printtext('EMM7021I');
    }

}

#
# Parsing the IP-Address for IPv4 and IPv6.
#
# On a list with IP addresses and port, the port get extract from the address.
sub parsingIpAddress {

    my $inputIpAddress = shift;
    my $defaultPort    = shift;
    my ($IPAddress, $IPPort);


    # IPv6 Address in brackets with port (e.g. [0000:0000::0001]:6740)
    if (index($inputIpAddress,']') > 0) {
       ($IPAddress,$IPPort) = split(']',$inputIpAddress);

       $IPAddress =~ s/\[//;          # Remove existing '[' from IPAdress
       $IPPort    =~ s/\://;          # Remove existing ':' from Port
    }
    # IPv6 Address without port, multiple ':' (e.g. ::1)
    elsif (index($inputIpAddress,':') < rindex($inputIpAddress,':')) {
       $IPAddress = $inputIpAddress;
       $IPPort    = '';
    }
    # IPv4 Address
    else {
       # Extract the port information (if specified). If none is given,
       # the port information in $Ermm::CLI::Conf::port will be used.
       ($IPAddress, $IPPort) = split(':', $inputIpAddress);
    }

    # Remove any leading/trailing whitespace
    $IPAddress =~ s/^\s*//;
    $IPAddress =~ s/\s*$//;

    if ($IPPort eq "" || (!defined $IPPort)) {
       $IPPort = $defaultPort;
    }
    else {
       $IPPort =~ s/^\s*//;
       $IPPort =~ s/\s*$//;
    }

    return $IPAddress, $IPPort;
}

#
# EBCDICtoASCII converter
#
sub EBCDICtoASCII() {
        my $line = shift;
        my $trTable =
    '\x00\x01\x02\x03\x9C\x09\x86\x7F\x97\x8D\x8E\x0B\x0C\x0D\x0E\x0F' .
    '\x10\x11\x12\x13\x9D\x0A\x08\x87\x18\x19\x92\x8F\x1C\x1D\x1E\x1F' .
    '\x80\x81\x82\x83\x84\x85\x17\x1B\x88\x89\x8A\x8B\x8C\x05\x06\x07' .
    '\x90\x91\x16\x93\x94\x95\x96\x04\x98\x99\x9A\x9B\x14\x15\x9E\x1A' .
    '\x20\xA0\xE2\xE4\xE0\xE1\xE3\xE5\xE7\xF1\xA2\x2E\x3C\x28\x2B\x7C' .
    '\x26\xE9\xEA\xEB\xE8\xED\xEE\xEF\xEC\xDF\x21\x24\x2A\x29\x3B\x5E' .
    '\x2D\x2F\xC2\xC4\xC0\xC1\xC3\xC5\xC7\xD1\xA6\x2C\x25\x5F\x3E\x3F' .
    '\xF8\xC9\xCA\xCB\xC8\xCD\xCE\xCF\xCC\x60\x3A\x23\x40\x27\x3D\x22' .
    '\xD8\x61\x62\x63\x64\x65\x66\x67\x68\x69\xAB\xBB\xF0\xFD\xFE\xB1' .
    '\xB0\x6A\x6B\x6C\x6D\x6E\x6F\x70\x71\x72\xAA\xBA\xE6\xB8\xC6\xA4' .
    '\xB5\x7E\x73\x74\x75\x76\x77\x78\x79\x7A\xA1\xBF\xD0\x5B\xDE\xAE' .
    '\xAC\xA3\xA5\xB7\xA9\xA7\xB6\xBC\xBD\xBE\xDD\xA8\xAF\x5D\xB4\xD7' .
    '\x7B\x41\x42\x43\x44\x45\x46\x47\x48\x49\xAD\xF4\xF6\xF2\xF3\xF5' .
    '\x7D\x4A\x4B\x4C\x4D\x4E\x4F\x50\x51\x52\xB9\xFB\xFC\xF9\xFA\xFF' .
    '\x5C\xF7\x53\x54\x55\x56\x57\x58\x59\x5A\xB2\xD4\xD6\xD2\xD3\xD5' .
    '\x30\x31\x32\x33\x34\x35\x36\x37\x38\x39\xB3\xDB\xDC\xD9\xDA\x9F' ;

        eval '$line =~ tr/\x00-\xFF/' . $trTable . '/';

        return $line;
 }

#
# ASCIItoEBCDIC converter
#
sub ASCIItoEBCDIC() {
        my $line = shift;
        my $trTable =
    '\x00\x01\x02\x03\x9C\x09\x86\x7F\x97\x8D\x8E\x0B\x0C\x0D\x0E\x0F' .
    '\x10\x11\x12\x13\x9D\x0A\x08\x87\x18\x19\x92\x8F\x1C\x1D\x1E\x1F' .
    '\x80\x81\x82\x83\x84\x85\x17\x1B\x88\x89\x8A\x8B\x8C\x05\x06\x07' .
    '\x90\x91\x16\x93\x94\x95\x96\x04\x98\x99\x9A\x9B\x14\x15\x9E\x1A' .
    '\x20\xA0\xE2\xE4\xE0\xE1\xE3\xE5\xE7\xF1\xA2\x2E\x3C\x28\x2B\x7C' .
    '\x26\xE9\xEA\xEB\xE8\xED\xEE\xEF\xEC\xDF\x21\x24\x2A\x29\x3B\x5E' .
    '\x2D\x2F\xC2\xC4\xC0\xC1\xC3\xC5\xC7\xD1\xA6\x2C\x25\x5F\x3E\x3F' .
    '\xF8\xC9\xCA\xCB\xC8\xCD\xCE\xCF\xCC\x60\x3A\x23\x40\x27\x3D\x22' .
    '\xD8\x61\x62\x63\x64\x65\x66\x67\x68\x69\xAB\xBB\xF0\xFD\xFE\xB1' .
    '\xB0\x6A\x6B\x6C\x6D\x6E\x6F\x70\x71\x72\xAA\xBA\xE6\xB8\xC6\xA4' .
    '\xB5\x7E\x73\x74\x75\x76\x77\x78\x79\x7A\xA1\xBF\xD0\x5B\xDE\xAE' .
    '\xAC\xA3\xA5\xB7\xA9\xA7\xB6\xBC\xBD\xBE\xDD\xA8\xAF\x5D\xB4\xD7' .
    '\x7B\x41\x42\x43\x44\x45\x46\x47\x48\x49\xAD\xF4\xF6\xF2\xF3\xF5' .
    '\x7D\x4A\x4B\x4C\x4D\x4E\x4F\x50\x51\x52\xB9\xFB\xFC\xF9\xFA\xFF' .
    '\x5C\xF7\x53\x54\x55\x56\x57\x58\x59\x5A\xB2\xD4\xD6\xD2\xD3\xD5' .
    '\x30\x31\x32\x33\x34\x35\x36\x37\x38\x39\xB3\xDB\xDC\xD9\xDA\x9F' ;

        eval '$line =~ tr/' . $trTable . '/\x00-\xFF/';
        return $line;
 }

sub L10N_init
{
	unless ($lh = nls::cli_L10N -> get_handle('cli_'.$Ermm::CLI::Conf::language.'_'.$Ermm::CLI::Conf::country)) {
          if (($Ermm::CLI::Conf::language eq 'en') && ($Ermm::CLI::Conf::country eq 'us')) {
             die "Error loading default NLS module cli_en_us!\n";
          }
          else {
             print "Error loading NLS module cli\_$Ermm::CLI::Conf::language\_$Ermm::CLI::Conf::country! Loading default NLS module...\n\n";
             $lh = nls::cli_L10N -> get_handle('cli_en_us') || die "Error loading default NLS module cli_en_us!\n";
          }
        }
	$lh -> fail_with("lookup_failure_handler");
}

sub printtext (@) {
	print( $lh -> maketext(@_) );
}

sub gettext (@) {
	$lh -> maketext(@_);
}

1;
