#!/usr/bin/perl -w
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#-----------------------------------------------------------------------
#
# @version 1.3 - 12/01/24 01:54:47
#
# @brief
#
# Change activities:
#     - 2005-05-09: Initial version
#
#-----------------------------------------------------------------------

package Ermm::CLI::Respond;
use Getopt::Long;
use Term::ReadLine;
use Ermm::CLI::Conf;
use Ermm::CLI::Main;

$synopsis =
  "respond [ -mount | -unmount | -move | -slotupdt | -driveupdt | -remove | -eject | -inject ]".
  " [ -slot <name> | -drive <name> | -pcl <name> ]".
  " [ -match <c> | -all | -client <c> | -text <match> | <request-ids>]".
  " [ -na | -v | -help, -?]";

sub respond {

    #
    # process command line options
    #
    my ($match, $type, $help, $acceptString, $reqids, $r,
        $mount, $unmount, $move, $slotupdt, $driveupdt, $eject, $inject,
        $slot, $pcl, $drive, $all, $text, $client, $noAutoAccept);

    $result = GetOptions ("v"        => \$Ermm::CLI::Conf::verbose, # flag
                          "match=s"  => \$match, # string
                          "type=s"   => \$type,  # string
                          "client=s" => \$client,# string
                          "text=s"   => \$text,  # string
                          "all"      => \$all,   # flag

                          "na"       => \$noAutoAccept,   # flag

                          "mount"    => \$mount,
                          "unmount"  => \$unmount,
                          "move"     => \$move,
                          "slotupdt" => \$slotupdt,
                          "remove"   => \$remove,
                          "driveupdt"=> \$driveupdt,
                          "eject"    => \$eject,
                          "inject"   => \$inject,

                          "slot=s"   => \$slot,
                          "pcl=s"    => \$pcl,
                          "drive=s"  => \$drive,

                          "help|?"   => \$help);   # flag

    #
    # check if we should print usage infos
    #
    if ($result != 1 || $help == 1) {
        Ermm::CLI::Main::printtext('_USAGE_RESPOND', $synopsis);
        exit 1;
    }

    #
    # build send string
    #
    my $sendString = "respond task[\"$Ermm::CLI::Conf::taskId\"]";

    if($#ARGV > 0) {
        foreach (1 .. $#ARGV) {
            $reqids .= " \"$ARGV[$_]\"";
        }
        $sendString .= " reqid[$reqids]";
    }

    if ($all) {
        $matchString = "regex(\".*\" Request.\"RequestId\")";
    }
    elsif ($match) {
        $matchString = "$match";
    }
    elsif ($client) {
        $matchString = "streq(Request.\"RequestingClient\" ".
            "\"$client\")";
    }
    elsif ($text) {
        $matchString = "regex(\".*$text.*\" Request.\"RequestText\")";
    }

    my $message_id;
    my @message_args;

    if ($mount) {
        $message_id="EMM4M10R"; # Mount completed

    }
    elsif ($unmount) {
        $message_id="EMM4M20R"; # Unmount completed

    }
    elsif ($move) {
        $message_id="EMM4M30R"; # Move completed

    }
    elsif ($slotupdt) {
        if ($slot and $pcl) {
            $message_id="EMM4M40R"; # Slot with cartridge
            push @message_args, 'slot', $slot;
            push @message_args, 'pcl', $pcl;
        }
        elsif ($slot) {
            $message_id="EMM4M41R"; # Empty slot
            push @message_args, 'slot', $slot;
        }
        else {
            $message_id="EMM4M42R"; # No more slot updates
        }

    }
    elsif ($driveupdt) {
        if($drive and $pcl) {
            $message_id="EMM4M60R"; # Drive with cartridge
            push @message_args, 'drive', $drive;
            push @message_args, 'pcl', $pcl;
        } elsif($drive) {
            $message_id="EMM4M61R"; # Empty drive
            push @message_args, 'drive', $drive;
        } else {
            $message_id="EMM4M62R"; # No more drive updates
        }
    }
    elsif ($remove) {
        $message_id = "EMM4M50R"; # Cartridge removal completed.

    }
    elsif ($eject) {
        $message_id = "EMM4N10R"; # Eject for automatic libraries

    }
    elsif ($inject) {
        $message_id = "EMM4N20R"; # Inject for automatic libraries

    }
    else {
        Ermm::CLI::Main::printtext('EMM7010E');
        Ermm::CLI::Main::printtext('_USAGE_RESPOND', $synopsis);
        exit 1;
    }

    if ($matchString) {
        $sendString .= "match[$matchString]";
    }
    $sendString .= " message[id[\"IBM\" \"TSLM\" \"$message_id\"]";
    if (@message_args) {
        $sendString .= " arguments [";
        foreach (@message_args) {
            $sendString .= " \"";
            $sendString .= $_;
            $sendString .= "\"";
        }
        $sendString .= "]";
    }
    $sendString .= "];";

    #
    # connect to TSLM server
    #
    Ermm::CLI::Main::connect();

    if (!$na) {
        #
        # automatically accept all matching requests in pending
        #
        $matchString = "and($matchString streq(Request.\"RequestState\" \"pending\"))";
        if ($reqId) {
            $matchString = "and($matchString or(";
            foreach $r (@reqid) {
                $matchString .= "streq(Request.\"RequestId\" \"$r\")";
            }
        }
        $acceptString = "accept task[\"$Ermm::CLI::Conf::taskId\"] ".
            "match[$matchString];";

        #
        # send command to server
        #
        Ermm::CLI::Main::send($acceptString);

        #
        # receive response from server; but don't exit on error
        #
        $response = Ermm::CLI::Main::getResponse(1);
    }
    #
    # send command to server
    #
    Ermm::CLI::Main::send($sendString);

    #
    # receive response from server
    #
    $response = Ermm::CLI::Main::getResponse();

}

1;
