#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#-----------------------------------------------------------------------
#
# @version 1.3 - 12/01/26 01:04:51
#
# @brief Simple input functionality based on POSIX termios
#
# Perl input functions for TSLM.
# The implementation is based on the POSIX termios standard
# and only works on POSIX systems (AIX, Linux, ...)
#
# Change activities:
#     - 2004-10-21: Initial version
#
#-----------------------------------------------------------------------

package ErmmInput;

use POSIX qw(:termios_h);

# Autoflush stdout, this helps getting the prompt on AIX
$|=1;

# Input function that clears STDIN and reads only one character
# Implemented only for POSIX systems
sub inputCharacter {
    my ($prompt) = @_;

    my ($original, $vtime) = charMode();

    # Discard any pending input.
    my $pending;
    do {
        my($input, $number) = '';
        vec($input, fileno(STDIN), 1) = 1;
        $pending = select($input,undef,undef,0);
        getc() if $pending;
    } while($pending);

    print $prompt;

    # Now read the character
    my $char;
    sysread(STDIN, $char, 1);

    previousMode($original, $vtime);

    # Now printing should work correctly again
    print $char . "\n";

    return $char;
}

# Put stdin in non-canonical mode to get single characters.
# The value of the c_lflag and the VTIME are returned to allow putting the terminal
# back into the previous mode.
sub charMode {
    my $term = POSIX::Termios->new();
    my $stdin = fileno(STDIN);
    $term->getattr( $stdin );

    # Save current terminal settings
    my $original = $term->getlflag();
    my $vtime = $term->getcc(VTIME);

    # Disable canonical mode to get characters without buffering
    # and disable echoing of characters
    $term->setlflag($original & (~ICANON) & (~ECHO) & (~ECHOK) );

    # Set timeout to minimum value.
    # This seems to be requires for AIX.
    $term->setcc(VTIME, 1);

    # Write settings to stdin
    $term->setattr($stdin, TCSANOW);

    return ($original, $vtime);
}

sub previousMode {
    my ($original, $vtime) = @_;

    my $term = POSIX::Termios->new();
    my $stdin = fileno(STDIN);

    $term->getattr( $stdin );

    # Return terminal to previous state
    $term->setcc(VTIME, $vtime);
    $term->setlflag($original);

    # Write settings to stdin
    $term->setattr($stdin, TCSANOW);
}

1;
