#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#-----------------------------------------------------------------------
#
# @version 1.3 - 12/01/26 01:04:52
#
# @brief Perl interface to ermmtool
#
# This module wraps the calls to ermmtool for scripts that want
# to use it as interface to TSLM
#
# Change activities:
#     - 2004-10-21: Initial version
#
#-----------------------------------------------------------------------

package ErmmTool;

use lib '.';
use nls::ts_L10N;

sub new {
    my ($class, $ermmtool, $ermmapp, $ermmai, $ermmauth, $ermmpass, $ermmserver, $ermmport, $debug) = @_;
    my $self = {
        '_ermmtool'     => $ermmtool,
        '_application'  => $ermmapp,
        '_instance'     => $ermmai,
        '_auth'         => $ermmauth,
        '_password'     => $ermmpass,
        '_server'       => $ermmserver,
        '_port'         => $ermmport,
        '_debug'        => $debug
    };
    bless($self, $class);
    return $self;
}

# Get all volumes, these are the volumes visible to the
# application used for accessing TSLM
# Return 0 on success and -1 on error.
sub volumes {
    my ($self, $volumes) = @_;

    $self->_ls("vol");
    foreach my $line ( @{$self->{_rawmsg}} ) {

        if ($line =~ /ECL1000I:.*/) {
            $$volumes = {};
            return 0;
        } elsif ($line =~ /ECL\d{4}\w:.*/) {
            $$volumes = {};
            return -1;
        } elsif ($line =~ /error: .*/) {
            $$volumes = {};
            return -1;
        } elsif ($line =~ /EMM7009I:.*/) {
            $$volumes = {};
            return 0;
        }
        my ($volume, $cartridge, $application, $mountedLast) = split /!/, $line;
        $$volumes->{$volume} = { 'Cartridge' => $cartridge,
                                'MountedLast' => $mountedLast };
    }
    return 0;
}

sub unmount {
    my ($self, $volume) = @_;

    my $unmountexit = $self->_system("unmount $volume");

    if($unmountexit == 0) {
        printtext('UNMNT_SUCCEEDED', $volume);
    } else {
        printtext('UNMNT_FAILED', $volume);
    }
}

sub alloc {
    my ($self, $volume) = @_;

    my $cartregex = '^' . $volume;
    my $cartridges = $self->cartridges($cartregex);

    my $number = keys %$cartridges;
    if($number == 0 ) {
        ErmmTool::printtext('SYNCMSG1006', $volume);
    }

    foreach my $cartridge(keys %$cartridges) {
        my $allocexit = $self->_system("alloc -pcl $cartridge $volume");
        if($allocexit == 0) {
            ErmmTool::printtext('ALLOC_SUCCEEDED', $volume, $cartridge);
            # Alloc has succeeded, so stop here
            last;
        } else {
            ErmmTool::printtext('ALLOC_FAILED', $volume, $cartridge);
        }
    }
}

sub dealloc {
    my ($self, $volume) = @_;
    my $application = $self->{_application};
    my $deallocexit = $self->_system("dealloc $volume -app $application");

    if($deallocexit == 0) {
        ErmmTool::printtext('DEALLOC_SUCCEEDED', $volume);
    } else {
        ErmmTool::printtext('DEALLOC_FAILED', $volume);
    }
}

sub cartridges {
    my ($self, $regex) = @_;

    my $cartridges = {};
    $self->_ls("cart -state allocatable -regex $regex");
    foreach my $line ( @{$self->{_rawmsg}} ) {
        my ($pcl, $type) = split /!/, $line;
        if(defined $pcl and defined $type) {
            $cartridges->{$pcl} = { 'Type' => $type };
        }
  }

  return $cartridges;
}

# Return 0 on success and -1 on error.
sub mounts {
    my ($self, $volumes) = @_;

    $self->_ls("ml -l") ;
    foreach my $line ( @{$self->{_rawmsg}} ) {
        if($self->{_debug}) {
            printtext('SYNCMSG1007', $line);
        }

        if ($line =~ /ECL1000I:.*/) {
            $$volumes = {};
            return 0;
        } elsif ($line =~ /ECL\d{4}\w:.*/) {
            $$volumes = {};
            return -1;
        }


        my ($mlid, $volume, $cartridge, $drive, $application,
            $handle, $DM, $capability) = split /!/, $line;
        if(defined $volume and $application eq $self->{_application}) {
            $$volumes->{$volume} = { 'MountLogicalId' => $mlid,
                                     'Cartridge'      => $cartridge,
                                     'Drive'          => $drive,
                                     'Handle'         => $handle,
                                     'DriveManager'   => $DM,
                                     'Capability'     => $capability };
        }
    }

    return 0;
}

# Execute one ermmtool command with system().
# Only the return value is of interest.
sub _system {
  my ($self, $command) = @_;

  my $syscmd = "$self->{_ermmtool} "
      . "-c $self->{_application} -i $self->{_instance} -am $self->{_auth} -p $self->{_password} -s $self->{_server}".($self->{_port}? ":$self->{_port}":"")
      . " $command";

  if($self->{_debug}) {
    printtext('SYNCMSG1008', $syscmd);
  }

  system($syscmd);

  if ($? == -1) {
    die printtext('SYNCMSG1009', $!);
  }
  elsif ($? & 127) {
    die printtext('SYNCMSG1010', ($? & 127), ($? & 128));
  }

  my $sysexit = $?>>8;

  return $sysexit;
}

# Get last response.
sub getRawMsg {
    my ($self) = @_;
    return @{$self->{_rawmsg}};
}

# Run ermmtool ls* command and return output as array with one line per array.
sub _ls {
    my ($self, $command) = @_;

    my $cmd = "$self->{_ermmtool} "
        . "-c $self->{_application} -i $self->{_instance} -am $self->{_auth} -p $self->{_password} -s $self->{_server}".($self->{_port}? ":$self->{_port}":"")
        . " ls$command -fmt delim=! -hdr off";

    if($self->{_debug}) {
        printtext('SYNCMSG1011', $cmd);
    }

    open my $out, "$cmd 2>&1 |" or die printtext('SYNCMSG1012', $cmd);
    my @output;
    while(<$out>) {
        chomp;
        push @output, $_;
        if($self->{_debug}) {
            printtext('SYNCMSG1013', $_);
        }
    }
    close($out);
    $self->{_rawmsg} = \@output;
}

# Send lslib command to ermmtool to check if MediaManager is running
sub ermmStopped {
    my ($self)=@_;
    $self->_ls("lib");
    foreach my $line ( @{$self->{_rawmsg}} ) {
       if ($line=~/EMM7002E:.*/){
          return -1;
       }
    }
    return 0;
}

sub L10N_init
{
   my $lng = shift;
   my $cntry = shift;
   $lh = nls::ts_L10N -> get_handle("ts\_$lng\_$cntry") || die "Error loading customized NLS module ts\_$lng\_$cntry!\n";
   $lh -> fail_with("lookup_failure_handler");
}

sub printtext (@) { 
   print( $lh -> maketext(@_) );
}

1;
