#!/usr/bin/perl -w
#----------------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Integrated Removable Media Manager for the Enterprise on System z
# 5655-S57
#
# (C) Copyright IBM Corp. 2004, 2007 All Rights Reserved
#
# US Government Users Restricted Rights - Use,  duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#----------------------------------------------------------------------------
#
# @version 1.1 - 10/03/23 12:42:48
#
# @brief genLogFiles - Setup and handle multiple logfile versions.
#                      Write the passed value into the active logfile.
#
#----------------------------------------------------------------------------

package genLogFiles;

use strict;
use IO::Handle;

my $logFile         = "";
my $logFileSize     = 1024*1024;
my $logFileVersion  = 1;
my $maxFileVersions = 8;
my $verbose         = 1;
my $versionInfo     = "";

#------------------------------------------------------------------------------
#
# Set the log file values
#
# Keywords              : logFileName, logFileSize, maxFileVersions, verbose,
#                         printVersionInfo
#
# Sample to pass values : setLogFileValue(logFileName      => "log/myLogFile",
#                                         logFileSize      => 512*512,
#                                         maxFileVersions  => 5,
#                                         verbose          => 1,
#                                         printVersionInfo => 'version-1.1.0');
#------------------------------------------------------------------------------
sub setLogFileValue {

    my %parameterlist = @_;

    #
    # Split parameterlist into his variables
    #
    if (defined $parameterlist{logFileName}) {
	$logFile = $parameterlist{logFileName};
    }
    if (defined $parameterlist{logFileSize}) {
	$logFileSize = $parameterlist{logFileSize};
    }
    if (defined $parameterlist{maxFileVersions}) {
	$maxFileVersions = $parameterlist{maxFileVersions};
    }
    if (defined $parameterlist{verbose}) {
	$verbose = $parameterlist{verbose};
    }
    if (defined $parameterlist{printVersionInfo}) {
	$versionInfo = $parameterlist{printVersionInfo};
    }

    if ($logFile eq '') {
       print "The name of the Logfile needs to be specified on setLogFileValue\n";
       return 1;
    }

    if ($verbose == 1) {
       openLastLogFile($logFile);
    }

    return 0;
}


#--------------------------------------------------------------------------
#
# Print the logfile
#
#--------------------------------------------------------------------------
sub logprint {

    if ($verbose == 1) {

       my ($sec, $min, $hour, $mday, $mon, $year) = localtime();
       my $t = sprintf("%04d-%02d-%02dT%02d:%02d:%02d", $year+1900, $mon+1,
                       $mday, $hour, $min, $sec);

       if ($logFile eq "") {
          print("$t:", @_, "\n");
       }
       else {
          print(LOG "$t:", @_, "\n");
          my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size,
              $atime, $mtime, $ctime, $blksize, $blocks) = stat(LOG);

          if ($size > $logFileSize) {
            if (++$logFileVersion > $maxFileVersions) {
               $logFileVersion = 1;
            }
            close(LOG);
            openLogFile($logFile, 1);
          }
       }
    }
}

#--------------------------------------------------------------------------
#
# Open the last logfile
#
#--------------------------------------------------------------------------
sub openLastLogFile {

    my $fname = shift;
    my $ret;

    for (my $i = 1; $i <= $maxFileVersions; $i++) {
        $logFileVersion = $i;
        $ret = openLogFile($fname, 0);

        if (!defined $ret) {
           print "Unable to open file $fname\n";
           exit 1;
        }

        my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size,
            $atime, $mtime, $ctime, $blksize, $blocks) = stat(LOG);

        if ($size < $logFileSize) {
            printVersionInfo();
            return $ret;
        }
        close(LOG);
    }
    $logFileVersion = 1;
    return openLogFile($fname, 1);     # Open a new logfile
}

#--------------------------------------------------------------------------
#
# Open a new logfile
#
#--------------------------------------------------------------------------
sub openLogFile {

    my $lf = shift;
    my $trunc = shift;
    my ($fname, $ret, $baseName);

    if ($lf =~ /([^\.]*)\.(.*)/) {
        $baseName = $1;
        $fname = sprintf("%s.%02d.log", $1, $logFileVersion);
    }
    else {
        $baseName = $lf;
        $fname = sprintf("%s.%02d.log", $lf, $logFileVersion);
    }
    # Open a new logfile
    if ($trunc == 1) {
        $ret = open(LOG, ">$fname");
        printVersionInfo();
    }
    # Reopen a existing logfile
    else {
        $ret = open(LOG, ">>$fname");
    }
    LOG->autoflush();

    return $ret;
}

#--------------------------------------------------------------------------
#
# Returns the last open logfile
#
#--------------------------------------------------------------------------
sub getLastLogFile {

    my $fname = shift;
    my $ret;

    for (my $i = 1; $i <= $maxFileVersions; $i++) {
        $logFileVersion = $i;
        $ret = openLogFile($fname, 0);

        my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size,
            $atime, $mtime, $ctime, $blksize, $blocks) = stat(LOG);

        if ($size < $logFileSize) {
            if ($fname =~ /([^\.]*)\.(.*)/) {
               $fname = sprintf("%s.%02d.log", $1, $logFileVersion);
            }
            return $fname;
        }
        close(LOG);
    }
    if ($fname =~ /([^\.]*)\.(.*)/) {
       $fname = sprintf("%s.%02d.log", $1, $logFileVersion);
    }
    return $fname;
}

#--------------------------------------------------------------------------
#
# Print the version Info to the logfile
#
#--------------------------------------------------------------------------
sub printVersionInfo {

    my ($sec, $min, $hour, $mday, $mon, $year) = localtime();
    my $t = sprintf("%04d-%02d-%02dT%02d:%02d:%02d", $year+1900, $mon+1,
                    $mday, $hour, $min, $sec);

    if ($versionInfo ne "") {
        print(LOG "$t:$versionInfo\n");
    }
}
1;
