#!/usr/bin/perl
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2006, 2012 All Rights Reserved
#-----------------------------------------------------------------------
#
# @version 1.3 - 12/01/26 00:50:44
#
# @brief eRMMCom.pm - TSLM MediaManager - Client Communication Module
#
#-----------------------------------------------------------------------

package eRMMCom;

use Socket;
use constant BASE64 => do{ eval{ require MIME::Base64 }};

use libbal::eRMMConfig;
use libbal::LIBBALVersion;
#
# private variables
#

#
# connect to TSLM server
#
sub connect {
    my($iaddr, $paddr, $proto, $line);

    $iaddr = inet_aton($eRMMConfig::mmhostname) or
        die eRMMConfig::printtext("ECL1000E", $eRMMConfig::mmhostname);
    $paddr = sockaddr_in($eRMMConfig::mmportnumber, $iaddr);
    $proto = getprotobyname('tcp');

    #
    # create a socket
    #
    socket(SOCK, PF_INET, SOCK_STREAM, $proto) or
        die eRMMConfig::printtext("ECL1001E", $!);

    setsockopt(SOCK, SOL_SOCKET, SO_KEEPALIVE, 1);

    #
    # and try to connect to the TSLM MediaManager
    #
    connect(SOCK, $paddr) or
        die eRMMConfig::printtext("ECL1002E", $eRMMConfig::mmhostname, $!);

    if ($eRMMConfig::mmclientauthmethod =~ m/pwenc/i) {
        if (BASE64) {
            $eRMMConfig::mmclientpassword =
                getEncryptedPassword($eRMMConfig::mmclientapplication,
                                     $eRMMConfig::mmclientappinstance);
        }
        else {
            eRMMTrace::addTraceInfo("can't use pwenc without MIME::Base64.pm. Install perl 5.8.0 or greater");
            close(SOCK);
            exit 1;
        }
    }

    #
    # start session and authenticate ourself
    #
    my $hello_string = "";
    if ($LIBBALVersion::LIBBALVERSION eq "LIBBALVERSIONSTRING") {
	$hello_string = "hello client[\"$eRMMConfig::mmclientapplication\"] ".
        " instance[\"$eRMMConfig::mmclientappinstance\"] ".
        " password[\"$eRMMConfig::mmclientpassword\"] ".
        " language[\"mmp\"];\n";
    }
    else {
	$hello_string = "hello client[\"$eRMMConfig::mmclientapplication\"] ".
        " instance[\"$eRMMConfig::mmclientappinstance\"] ".
        " password[\"$eRMMConfig::mmclientpassword\"] ".
        " language[\"mmp\"] clientversion [\"$LIBBALVersion::LIBBALVERSION\"];\n";

    }
    send SOCK, $hello_string, 0;

    $line = <SOCK>;

    if (!defined $line) {
        eRMMTrace::addTraceInfo("unable start session");;
        close(SOCK);
        exit 1;
    }

    if (!defined $line || $line =~ /unwelcome/) {
        eRMMTrace::addTraceInfo("unable start session: " . $line);
        close(SOCK);
        exit 1;
    }
}

#
# send a command string to the TSLM server
#
sub send {
    my $line = shift;
      send SOCK, $line, 0;
}

#
# waiting for accept and success responses
#
sub getResponse {
    my $dontExitOnError = shift;
    my $line = '';
    my $error = 0;
    my $status;

    do {
        $line = <SOCK>;
        if (defined($line) && $line) {
            # remove leading and trailing space chars
            $line =~ s/^\s+//;
            $line =~ s/\s+$//;
        }
        else {
            eRMMTrace::addTraceInfo("error reading response:" . $! );
            return "error";
        }
    } while (length($line) == 0);


    if ($line =~ /^\s*response\s+unacceptable.*/) {
        eRMMTrace::addTraceInfo("command unacceptable");
        return "error";
    }

    do {
        $line = <SOCK>;
        if (defined($line) && $line) {
            # remove leading and trailing space chars
            $line =~ s/^\s+//;
            $line =~ s/\s+$//;
        }
        else {
            eRMMTrace::addTraceInfo("error reading response:" . $! );
            return "error";
        }
    } while (length($line) == 0);


    #
    # remove trailing ;
    #
    $line =~ s/;$//;

    $status = getResponseStatus($line);

    if (defined $dontExitOnError && $dontExitOnError) {
        return $line;
    }

    if (lc($status) eq "unacceptable") {
        eRMMTrace::addTraceInfo("command unacceptable");
        return "error";
    }
    elsif (lc($status) eq "cancelled") {
        eRMMTrace::addTraceInfo("command cancelled");
        return "error";
    }
    elsif (lc($status) eq "error") {
        my $errorCode = getResponseErrorCode($line);
        my $info = getResponseMessage($line);
        if ($info eq "") {
            eRMMTrace::addTraceInfo("error executing command:" . $errorCode );
        }
        else {
            eRMMTrace::addTraceInfo($info);
        }
        return "error";
    }

    return $line;
}

sub getResponseStatus {
    my $line = shift;

    if ($line =~ /^\s*response\s+unacceptable.*/) {
        return "unacceptable";
    }
    $line =~ /^\s*response\s+task\s*\[\"([^\"]*)\"\]\s*([a-zA-Z]+).*/;

    return $2;
}

sub getResponseErrorCode {
    my $line = shift;

    $line =~ /^\s*response\s+task\s*\[\s*\"([^\"]*)\"\s*\]\s*error\s*\[\s*\"([^\"]*)\"\s*\"([^\"]*)\"\s*\].*/;
    return "$3";
}

sub getResponseMessage {
    my $line = shift;

    $line =~ /.*loctext\s*\[\s*\"([^\"]*)\"\s*\"([^\"]*)\"\s*\].*/;
    return "$1";
}

sub getNotification {
    my $dontExitOnError = shift;
    my $line = '';
    my $error = 0;
    my $status;
    do {
        $line = <SOCK>;
        if (defined($line) && $line) {
            # remove leading and trailing space chars
            $line =~ s/^\s+//;
            $line =~ s/\s+$//;
        }
        else {
            eRMMTrace::addTraceInfo("error reading notification:" . $! );
            return "error";
        }
    } while (length($line) == 0);


    #
    # remove trailing ;
    #
    $line =~ s/;$//;

    $status = getResponseStatus($line);

    if (defined $dontExitOnError && $dontExitOnError) {
        return $line;
    }

    if (lc($status) eq "unacceptable") {
        eRMMTrace::addTraceInfo("command unacceptable");
        return "error";
    }
    elsif (lc($status) eq "cancelled") {
        eRMMTrace::addTraceInfo("command cancelled");
        return $line;
    }
    elsif (lc($status) eq "error") {
        my $errorCode = getResponseErrorCode($line);
        my $info = getResponseMessage($line);
        eRMMTrace::addTraceInfo($info);
        return "error";
    }

    return $line;
}

sub getKeyValueResults {
    my $line = shift;
    my @values;
    my $i = 0;
    my @objs = (split /\btext/, $line);

    for ($i = 1; $i <= $#objs; $i++) {
        $_ = $objs[$i];
        # remove leading and trailing space chars
        s/^\s+//;
        s/\s+$//;
        # remove leading and trailing brackets
        s/^\[//;
        s/\]$/ "tail"/;
        # remove leading and trailing quote
        s/^\"//;
        s/\"$//;
        # split results at " "
        push @{$values[$i-1]}, (split /\"\s+\"/) ;
        # remove tail
        pop @{$values[$i-1]};
    }
    return @values;
}

sub encrypt($) {
    my $s = shift;
    my $d = "";

    for (my $i = 0; $i < length($s); $i++) {
        $d .= chr(ord(substr($s, $i, 1)) ^ 0xA5);
    }
    return $d;
}


sub getEncryptedPassword() {
    my $app = shift;
    my $ai = shift;
    my $pw = "";
    my ($fapp, $fai, $fpw, $rest, $line);

    if (open(PWFILE, "/opt/IBM/ermm/conf/ermmsecurity.conf")) {
        while ($line = <PWFILE>) {
            ($fapp, $fai, $fpw, $rest) = split(":", $line);
            if ($fapp eq $app && $fai eq $ai) {
                return MIME::Base64::encode_base64(encrypt($fpw), "");
            }
        }
    }
    else {
        eRMMTrace::addTraceInfo("cannot open secure password file");
        exit 1;
    }
    return $pw;
}


#
# send message to Media Manager
#
# Arguments: Message id followed by severity, then key, value pairs for the arguments,
#            e.g. sendMessage(EMM1234, "DMHost", "127.0.0.1", "SerialNumber", "0815")
#
sub sendMessage {
  my ($id, $severity, @args) = @_;
  my $tid = eRMMConfig::getTaskId();
  my $cmd = qq/message task ["$tid"] who ["MM"] severity ["$severity"] message [ id [ "IBM" "TSLM" "$id" ] arguments [ /;
  foreach (@args) {
    $cmd .= qq/"$_" /;
  }
  $cmd .= qq/ ] ] ;\n/;

  eRMMCom::send($cmd);
  getResponse();
}

sub sendError {
	my ($data) = @_;
	sendMessage('EMM6002E', 'error',
                               'application', $eRMMConfig::mmclientapplication,
					 'instance', $eRMMConfig::mmclientappinstance,
                               'message', $data);
	return;
}
sub sendInfo {
	my ($data) = @_;
	sendMessage('EMM6001I', 'information',
                               'application', $eRMMConfig::mmclientapplication,
					 'instance', $eRMMConfig::mmclientappinstance,
                               'message', $data);
	return;
}


1;
